#! /usr/bin/env python

#######################################################
#
#  Draws a source distribution according to Gilli 2007
#
#
#  Author: N.Clerc/MPE
#
#
#  Created: 08/05/2012
#
#######################################################

import os,sys,time
import pyfits as pf
#import astropy.io.fits as pf
import numpy as np
import string
import subprocess
from scipy import interpolate,integrate
from math import log10,sqrt,pi,exp,cos,floor
import spectrotools as sptools
from scipy.special import erf
import cosmolopy.distance as cd
import mk_gilli_templates as gi
#from joblib import Parallel,delayed
import simput
import numpy.core.defchararray as defchar
import make_bkgpatch

def mydist_on_sphere(l1inp,b1inp,l2inp,b2inp):
  
#    ##
#    Formula for distance on sphere accurate over entire sphere
#    (Vincenty, Thaddeus, 1975). Input and output are in degrees.
#    
#    :param l1:
#    Longitude of first location on sphere
#    :type l1:
#    float
#    :param b1:
#    Latitude of first location on sphere
#    :type b1:
#    float
#    :param l2:
#    Longitude of second location on sphere
#    :type l2:
#    float
#    :param b2:
#    Latitude of second location on sphere
#    :type b2:
#    float
#    
#    :Examples:
#    
#    >>> from kapteyn.maputils import dist_on_sphere
#    >>> print dist_on_sphere(0,0, 20,0)
#    20.0
#    
#    >>> print dist_on_sphere(0,30, 20,30)
#    17.2983302106
#    
#   ##

    fac = np.pi / 180.0
    l1 = l1inp * fac; b1 = b1inp * fac; l2 = l2inp * fac; b2 = b2inp * fac
    dlon = l2 - l1
    aa1 = np.cos(b2)*np.sin(dlon)
    cosdlon = np.cos(dlon)
    dlon = 0
   
    aa2 = np.cos(b1)*np.sin(b2) - np.sin(b1)*np.cos(b2)*cosdlon
    a = np.sqrt(aa1*aa1+aa2*aa2)
    aa1 = 0
    aa2 = 0
   
    b = np.sin(b1)*np.sin(b2) + np.cos(b1)*np.cos(b2)*cosdlon
    cosdlon = 0
    
    d = np.arctan2(a,b)
    a = 0
    b = 0
    return d/fac



def calc_pi(gammaminlist,gammamaxlist,meangamma,sigmagamma):

#    #x
#    Simply integrates the gaussian distribution in bins defined by
#    gammaminlist[i] - gammamaxlist[i]
#
#    Returns a result so that sum(result) = 1
#    #

    nval=len(gammaminlist)
    result = np.zeros(nval)
    
    for i in range(nval):
        ai_plus = gammamaxlist[i]
        ai_minus = gammaminlist[i]
        bi_plus = (ai_plus-meangamma)/(sigmagamma*sqrt(2.))
        bi_minus = (ai_minus-meangamma)/(sigmagamma*sqrt(2.))
        pi = 0.5*( erf(bi_plus)-erf(bi_minus) )
        result[i] = pi

    norm = np.sum(result)
    # Gillis says it is normalized to 1
    return result/norm


#-------------------------------------------------

def hasinger(z,Lx):
#    """
#    Returns the LDDE of Hasinger 2005
#    lum052: input luminosity in h70**-2 erg/s
#    z: redshift
#    h70: value of h70
#
#    Result: dphi/dlogLX in h70**3 Mpc**-3
#
#    Notes:
#     (1) the normalization factor in eq. (3) should be computed for zc=0.1
#         strictly following the definition of A44.
#         However such a choice leads to small inconsistencies when comparing
#         the program output to Hasinger 2005 plots. We choose to compute
#         A44 at z=0.0 to perfectly reproduce fig. 4 of H05.
#    """


    ## Table 5 parameters
    A44 = 2.62e-7     # h70**3 Mpc**-3
    logLXstar = 43.94 # h70**-2 erg s**-1
    g1 = 0.87
    g2 = 2.57
    p1_44 = 4.7
    zc_44 = 1.42
    p2_44 = -1.5
    alpha = 0.21
    logLXc = 44.67
    b1 = 0.7
    b2 = 0.6

    ## Some secondary parameters
    if(logLXc >= 44):
        zc_0 = zc_44*(1e44/(10.**logLXc))**(-alpha)
    else:
        zc_0 = zc_44

    ## Compute evolution values eq (9)-(12)
    if(Lx <= 10.**logLXc):
        zc=zc_0*(Lx/10.**logLXc)**alpha
    else:
        zc=zc_0

    p1 = p1_44+b1*(log10(Lx)-44.)
    p2 = p2_44+b2*(log10(Lx)-44.)

    if(z <= zc):
        ed = (1+z)**p1
    else:
        ed = (1+zc)**p1 * ((1+z)/(1+zc))**p2

    ## Calculate norm of eq. (2)
    refzc = 0.0 # cf. note (1)
    f44 = 1./( (10.**(44.-logLXstar))**g1 + (10.**(44.-logLXstar))**g2 )
    if(refzc <= zc_44):
        ed_zref_l44 = (1.+refzc)**p1
    else:
        ed_zref_l44 = (1.+refzc)**p1 * ((1.+refzc)/(1.+zc_44))**p2

    norm = A44/(f44 * ed_zref_l44)

    ## Calculate at z=0 using this norm
    dphidL_0 = norm/((Lx/10.**logLXstar)**g1+(Lx/10.**logLXstar)**g2)

    return dphidL_0*ed

#-------------------------------------------------
def unobscured_integrand(logLx,z,cosmo):
    """
    Function returning (dN/dLogLx/dV)*(dVcom/dOmega/dz)

    * Needs an cosmo
    * logLx: log10(Lx) with Lx in ergs/s
    * z
    
    Result in sr**-1
    """

    h70 = cosmo["h"]*100./70.
    logLx_h70 = logLx+2*log10(h70)

    d2vdomdz = cd.diff_comoving_volume(z, **cosmo)
    
    return hasinger(z,10.**logLx_h70)*d2vdomdz*h70**3

#-------------------------------------------------
def obscured_integrand(logLx,z,cosmo):
    
    """
    Function returning (dN/dLogLx/dV)*(dVcom/dOmega/dz)
    Based on unobscured luminosity function, then applying
    eq. (4) og Gilli 2007

    * Needs a cosmo
    * logLx: log10(Lx) with Lx in ergs/s
    * z
    
    Result in sr**-1
    """
    Lx = 10.**logLx
    h70 = cosmo["h"]*100./70.

    logLx_h70 = logLx+2*log10(h70)

    # Cthin to Unobscured ratio
    R_S = 4.
    R_Q = 1.
    L_c = 10.**(43.5)
    RL = R_S*exp(-Lx/L_c)+R_Q*(1-exp(-Lx/L_c))
    
    d2vdomdz = cd.diff_comoving_volume(z, **cosmo)

    return RL*hasinger(z,10.**logLx_h70)*d2vdomdz*h70**3

#-------------------------------------------------
def obscured_integrand_gamma(logLx,z,cosmo,convsey,convqso,QSO_thresh):
    
    """
    Function returning (dN/dLogLx/dV)*(dVcom/dOmega/dz)
    Based on unobscured luminosity function, then applying
    eq. (4) og Gilli 2007, converted in the band.
    It fills up the band keV band with Compton-Thin sources.

    * Needs a cosmo
    * logLx: log10(Lx) with Lx in ergs/s, in the band keV band
    * z
    * gamma: index of slope
    * conv_dict a conversion dictionnary
    
    Result in sr**-1
    """

    ## band keV luminosity
    Lx = 10.**logLx
 
    # Conversion factor
    # Choose QSO or Sey
    # Inherent non-bijectivity (Gilli problem ?): I make the choice of Seyfert per default
    # through the check_seyqso function.
    ##
    #fsey=interpolate.interp1d(conv_dic["gamma"],conv_dic["sey"])
    #fqso=interpolate.interp1d(conv_dic["gamma"],conv_dic["qso"])
    #convsey = fsey(gamma)
    #convqso=fqso(gamma)

    l052sey = convsey*Lx
    l052qso = convqso*Lx
    mon_type = sptools.check_seyqso(l052sey,l052qso,QSO_thresh=QSO_thresh)
    
    if(mon_type == "sey"):
        Lx_soft = l052sey
    else:
        Lx_soft = l052qso
    #Lx_soft = l052sey ## Force Seyfert

    ## [0.5-2] keV luminosity
    logLxsoft = log10(Lx_soft)
    h70 = cosmo["h"]*100./70.


    # Cthin to Unobscured ratio
    # R_S = 4.
    # R_Q = 1.
    # L_c = 10.**(43.5)
    ratio = exp(-10.**(logLxsoft-43.5))
    RL = 4.*ratio+1.*(1.-ratio)

    d2vdomdz = cd.diff_comoving_volume(z, **cosmo)

    return RL*hasinger(z,Lx_soft*h70**2)*d2vdomdz*h70**3



#-------------------------------------------------
def invert_nhcumul(y,lognhmin,lognhmax,fracnh):
    
    """
    Provides the inverse of a cumulative histogram of Nh.
    The probability distribution is piecewise uniform:
     fractional probability is fracnh in bins defined by lognhmin,lognhmax
    """

    if(not ((y<1.)*(y>=0.))):
        print "Error, y must be in [0,1["
        sys.exit(1)

    cumsum_cj_plus= np.cumsum(fracnh)
    cumsum_cj_minus = np.concatenate( (np.array([0]),cumsum_cj_plus[0:-1]) )

    k = np.where( (y >= cumsum_cj_minus) * (y < cumsum_cj_plus))[0]
    res = (1./fracnh[k])*(y-cumsum_cj_minus[k]*(lognhmax[k]-lognhmin[k]))+lognhmin[k]

    return res


#-------------------------------------------------
def rough_maximum(func,xbounds,ybounds,gridx=50,gridy=50):
    """
    Computes values of func over a grid defined by x/y parameters.
    Returns the maximum value.

    x and y are linear inputs
    """  
    value=np.zeros((gridx,gridy))
    #print "  Finding maximum with gridx=%i and gridy=%i" % (gridx,gridy)

    for i in range(gridx):
        xval = xbounds[0]+(xbounds[1]-xbounds[0])*i/float(gridx)
        for j in range(gridy):
            yval = ybounds[0]+(ybounds[1]-ybounds[0])*j/float(gridy)
            value[i,j] = func(xval,yval)

    return value.max()

#-------------------------------------------------
def prepare_convfac_old(dirtemp,ming,maxg,ngs,band):
    
    """
    Calculates conversion factors for unabsorbed sources with a range of gammas.
    The result is L[soft]/L[band] for each gamma
    """

    gammagrid = np.linspace(ming,maxg,ngs)
    
    # Seyfert Conversion factor
    conv_sey = np.zeros(ngs)
    for i in range(ngs):
        ee,ss = gi.mk_gillimodel(dirtemp,gammagrid[i],1,0.0,10.**40.,bandlum=[0.5,2.],addstring='calibsey_',QSO_thresh=10**(43.5))
        conv_sey[i]   = sptools.convert_bandluminosity(band,1.0,ee,ss,[0.5,2.])

    # QSO Conversion factor
    conv_qso = np.zeros(ngs)
    for i in range(ngs):
        ee,ss = gi.mk_gillimodel(dirtemp,gammagrid[i],1,0.0,10.**50.,bandlum=[0.5,2.],addstring='calibqso_',QSO_thresh=10**(43.5))
        conv_qso[i]   = sptools.convert_bandluminosity(band,1.0,ee,ss,[0.5,2.])

    return {"gamma":gammagrid,"sey":conv_sey,"qso":conv_qso}

#-------------------------------------------------
def prepare_convfac(bankunabs,ming,maxg,ngs,band):
    
    """
    Calculates conversion factors for unabsorbed sources with a range of gammas.
    The result is L[soft]/L[band] for each gamma
    """

    gammagrid = np.linspace(ming,maxg,ngs)
    nhgrid = np.zeros(ngs)+1.
    zgrid = np.zeros(ngs)
    
    
    # Seyfert Conversion factor
    conv_sey = np.zeros(ngs)
    lgrid = np.zeros(ngs)+10.**(40.)
    ee,ss,b,b,b,b = gi.read_specbank(bankunabs,gammagrid,nhgrid,zgrid,lgrid,bandlum=[0.5,2.],QSO_thresh=10**(43.5))
    for i in range(ngs):
        conv_sey[i]   = sptools.convert_bandluminosity(band,1.0,ee[i],ss[i],[0.5,2.])

    # QSO Conversion factor
    conv_qso = np.zeros(ngs)
    lgrid = np.zeros(ngs)+10.**(50.)
    ee,ss,b,b,b,b = gi.read_specbank(bankunabs,gammagrid,nhgrid,zgrid,lgrid,bandlum=[0.5,2.],QSO_thresh=10**(43.5))
    for i in range(ngs):
        conv_qso[i]   = sptools.convert_bandluminosity(band,1.0,ee[i],ss[i],[0.5,2.])

    return {"gamma":gammagrid,"sey":conv_sey,"qso":conv_qso}




#-------------------------------------------------
def gen_positions(Nsources,area_def,geomtype=''):
    """
    Generates random (RA,DEC) positions for Nsources
    within the [ramin,ramax], [decmin,decmax] rectangular
    region.

    (To be updated to take into account correlation)

    UPDATES:
       - 26.11.2012: possibility to do the job in a projected circle instead of a RAxDEC box
    """

    #print "FUVL:",geomtype, Nsources
    
    if(geomtype == 'angularbox'):
        ramin = area_def[0][0]
        ramax = area_def[0][1]
        decmin = area_def[1][0]
        decmax = area_def[1][1]

        amin = ramin*pi/180.        # radians
        amax = ramax*pi/180.        # radians
        bmin = (90.-decmin)*pi/180. # radians
        bmax = (90.-decmax)*pi/180. # radians

        u = np.random.random(Nsources)
        v = np.random.random(Nsources)

        phi = amin+(amax-amin)*u
        
        vv = 0.5*(1+np.cos(bmax))*(1.-v)+0.5*(1.+np.cos(bmin))*v
        lambd = np.arccos(2.0*vv-1.)
        
        ra = phi*180./pi
        dec = 90.-(lambd*180./pi)

        
    if(geomtype == 'projcircle'):
        ra_cen = area_def[0]
        dec_cen = area_def[1]
        radius = area_def[2]

        #print 'HER',ra_cen,dec_cen,radius

        
        tot_area_sr = pi*(radius**2)*(pi/180.)**2.

        Nsphere = Nsources*(4*pi/tot_area_sr)
        Nsim = 0

        #print  tot_area_sr, 
        
        while(Nsim < Nsources):
           ra = 0
           dec = 0
           # 1. Methode bourrin: simulate the full sky, recommence si pas assez de points dans la region voulue
           Nsphere += 5*sqrt(Nsphere)
           print "-- gen_positions: Nsphere = %i" % Nsphere
           u = np.random.random(int(Nsphere))
           v = np.random.random(int(Nsphere))
           phi = 2*pi*u
           lambd = np.arccos(2.0*v-1.)
           u = 0
           v = 0
           ra = phi*180./pi
           dec = 90.-(lambd*180./pi)
           phi = 0
           lambd = 0
           # selection: remove "far away points"
           # 
           mydist = mydist_on_sphere(ra,dec,ra_cen,dec_cen)           
           ok = mydist < radius
           mydist = 0
           Nsim = ok.sum()
           print "   gen_positions: Simulated = %i" % Nsim

        # AGE HAS COMMENTED OUT #*180./pi
        # ra, dec already in degrees
        myra  =  ra[ok]#*180./pi
        mydec = dec[ok]#*180./pi
        ra = myra
        dec = mydec
        print myra
        print mydec
        difference = Nsim-Nsources
        print "gen_positions: Selected = %i" % len(myra)
        print "               Expected = %i (diff = %i)" % (Nsources,difference)
        # Clean randomly the list
        if(difference > 0):
           # Generate <difference> random integers in [0,Nsim-1] without replacement
           mylist = np.arange(Nsim)
           np.random.shuffle(mylist)
           suppress = mylist[:difference]
           # Suppress the points from our array
           ra = np.delete(ra,suppress)
           dec = np.delete(dec,suppress)
           print "gen_positions: Kept = %i" % len(ra)


    return ra,dec




#-------------------------------------------------
def gen_GRIDpositions(area_def,geomtype='', doffset=2./60.):
    """
    AGE: modified by age based on Nicolas' modules
    to generate a grid of RA, DEC positions within the
    FOV for calibration purposes. 
    within the [ramin,ramax], [decmin,decmax] rectangular
    region.

    doffset is the minimum distance in each of RA and DEC between
    positions in units of degrees. 

    (To be updated to take into account correlation)

    UPDATES:
       - 26.11.2012: possibility to do the job in a projected circle instead of a RAxDEC box
    """

    if(geomtype == 'angularbox'):

        ramin = area_def[0][0]
        ramax = area_def[0][1]
        decmin = area_def[1][0]
        decmax = area_def[1][1]

        ra_cen = ramin +  (ramax-ramin)/2.0
        dec_cen = decmin + (decmax-decmin)/2.0
        
    if(geomtype == 'projcircle'):
        ra_cen = area_def[0]
        dec_cen = area_def[1]
        radius = area_def[2]

        ramin = ra_cen - radius / np.cos(dec_cen * pi/180.)
        if(ramin<0):
            ramin=360.0+ramin
        ramax = ra_cen + radius / np.cos(dec_cen * pi/180.)   
        if(ramax>360):
            ramax=ramax-360.0
        
        decmin = dec_cen - radius
        if(decmin<-90):
            decmin=-90
        decmax = dec_cen + radius
        if(decmax>90):
            decmax=90

    #print geomtype
    #print ra_cen,dec_cen,radius
    #print ramin, ramax, decmin, decmax

        
    #amin = ramin*pi/180.        # radians
    #amax = ramax*pi/180.        # radians
    #bmin = (90.-decmin)*pi/180. # radians
    #bmax = (90.-decmax)*pi/180. # radians
    #acen = ra_cen*pi/180
    #bcen = (90.-dec_cen)*pi/180. # radians

    mydist_ra = mydist_on_sphere(ramin,ramax,dec_cen,dec_cen)
    mydist_dec = mydist_on_sphere(ra_cen,ra_cen,decmin,decmax)

    Nra_max = int(mydist_ra / doffset)
    Ndec_max = int(mydist_dec / doffset)

    #print Nra_max, Ndec_max, mydist_ra, mydist_dec
    
    ra=[]
    dec=[]
    for idec in range(Ndec_max):

        d=decmin + (float(idec) - 0.5 ) * doffset
                
        for ira in range(Nra_max):
            r=ramin + ira * (doffset)/np.cos(dec_cen * pi/180.0)
            if(r>360):
                r=r-360.0            
            if(r<0):
                r=360.0+r            
            #print r,d,ira,idec
            ra.append(r)
            dec.append(d)
            
    ra=np.asarray(ra)
    dec=np.asarray(dec)    

    #print ra[0:10]
    if(geomtype == 'projcircle'):
    #    print ra.shape, dec.shape
        mydist = mydist_on_sphere(ra,dec,ra_cen,dec_cen)
        ok = mydist < radius
        print "RADIUS IS",radius
    if(geomtype == 'angularbox'):
        mask1 = ra>ramin
        mask2 = ra<ramax
        mask3 = dec>decmin
        mask4 = dec<decmax
        mask = np.logical_and(mask1, maks2)
        mask = np.logical_and(mask, maks3)
        mask = np.logical_and(mask, maks4)
        ok = mask
        

    Nsim = ok.sum()
    print "   gen_positions: Simulated = %i" % Nsim
    myra  =  ra[ok]
    mydec = dec[ok]

    #print myra[0:10]
    #print mydec
    #sys.exit()
    return myra,mydec;



#-------------------------------------------------
def split_agnlist(dir,filename,listname,bkgname,area_def=[],area_type='',emin=0.1,emax=1000.,fluxlim=1e-16,skipBkg=False):

    """
    dir MUST be absolute path

    Splits a simulated AGN list into two parts :
     - above fluxlim, make a separate list of sources
     - below fluxlim, make a single spectrum and a flat image

    v0.1: * the flux is that of the "FLUX" column in simput Catalogue
          (no matter what the energy band is)
          * the spectral association is not performed (VERY BAD)
          * works only if the reference to spectra is given by names
    
    v0.2: possibility to define a projected circle or an angular box

    v0.3: possibility to skip the BKG construction

    Author: N. Clerc, 21.07.2012
    
    """

    #~ Check path
    if(dir[0] != '/'):
       ans = ''
       done = 0
       print "***split_agnlist: WARNING, does not seem to be an absolute path"
       while(done == 0):
          ans = raw_input(" Go ahead? (y/n)")
          if(ans == 'y'):
             done = 1
          elif(ans == 'n'):
             done = 2
          else:
             done = 0
       if(done == 2):
          sys.exit(1)
       

    #~ Check geonmetry
    if(area_type == 'angularbox'):
        ramin = area_def[0][0]*pi/180.
        ramax = area_def[0][1]*pi/180.
        decmin = area_def[1][0]*pi/180.
        decmax = area_def[1][1]*pi/180.
        racen = 0.5*(ramin+ramax)
        deccen = 0.5*(decmin+decmax)
        area_sr = (ramax-ramin)*(cos(pi/2.-decmax)-cos(pi/2.-decmin))
        area = 3600.*area_sr*(180./pi)**2 ## arcmin2
    elif(area_type == 'projcircle'):
        racen = area_def[0]
        deccen = area_def[1]
        radius = area_def[2]
        area = pi*radius**2
        area_sr = area*(pi/180.)**2.
        area *= 3600. ## arcmin2
    else:
        print "Error: must specify area_type"
        sys.exit(1)


    #~ Read SIMPUT catalogue
    agnlist = simput.readfile(dir+"/"+filename)

    #~ Make position selection
    if(area_type == 'angularbox'):
       okpos = (agnlist.catalogue.data["RA"] >= ramin)*(agnlist.catalogue.data["RA"] <= ramax)*(agnlist.catalogue.data["DEC"] >= decmin)*(agnlist.catalogue.data["DEC"] <= decmax)
    if(area_type == 'projcircle'):
       allra = agnlist.catalogue.data["RA"].copy()
       alldec = agnlist.catalogue.data["DEC"].copy()
       mydist = mydist_on_sphere(allra,alldec,racen,deccen)
       okpos = (mydist <= radius)
    print "Kept %i/%i after positional selection" % (np.sum(okpos),agnlist.catalogue.nsources)

    above = (agnlist.catalogue.data["FLUX"] >= fluxlim) * okpos   
    below = (agnlist.catalogue.data["FLUX"]  < fluxlim) * okpos   

    n_above = np.sum(above)
    n_below = np.sum(below)    

    #~ Reassociate spectra to sources
    # NOT PERFORMED, MUST BE CORRECTED
    # Here we assume that the list of sources is 1<->1 correspondance with list of spectra
    # In most cases, not true

    #~ Make source list f>flim
    subset = simput.simputobj(['catalogue','spectrum'],cat_columns=['SRC_NAME'],
                                 spec_columns=['NAME'],speclength=agnlist.spectrum.speclength)
    
    print " -> %i sources above %e ergs/s/cm2 (and within region)" % (n_above,fluxlim)


    nullcatflag = False

    if(n_above ==  0):
        n_above = 1
        nullcatflag = True

    if(n_above > 1):
        subset.catalogue.nsources = n_above
        for key in subset.catalogue.data.keys():
            subset.catalogue.data[key] = agnlist.catalogue.data[key][above]
    else: subset.catalogue.addSource(agnlist.catalogue.getSource(0))


    if(n_above > 1):
        subset.spectrum.nspectra = n_above
        for key in subset.spectrum.data.keys():
            if(key == "ENERGY" or key == "FLUX"):
                subset.spectrum.data[key] = agnlist.spectrum.data[key][above,:]
            else:
                subset.spectrum.data[key] = agnlist.spectrum.data[key][above]
    else:
        subset.spectrum.addSpectrum(agnlist.spectrum.getSpectrum(1))

    if(nullcatflag):
        if("FLUX" in subset.catalogue.data.keys()):
            subset.catalogue.data["FLUX"][0] = 0.

    subset.writeto(dir+"/"+listname)
    subset = 0

    #~ Construct background
    if(not skipBkg):
        flux_below = agnlist.catalogue.data["FLUX"][below]
        e_min_below = agnlist.catalogue.data["E_MIN"][below]
        e_max_below = agnlist.catalogue.data["E_MAX"][below]
        temp = defchar.split(agnlist.catalogue.data["SPECTRUM"][below],"'")
        specname_below = [x[1] for x in temp]

        e = np.logspace(log10(emin),log10(emax),num=agnlist.spectrum.speclength,base=10.0)
        s = np.zeros(agnlist.spectrum.speclength)

        print " -> %i sources below %e ergs/s/cm2 (and within region)" % (n_below,fluxlim)

        for i in range(n_below):
            print " > Constructing BKG spectrum %3.0f " % (100.*i/n_below),"percent complete    \r",
            okspec, = np.where(agnlist.spectrum.data["NAME"] == specname_below[i])
            okspec = okspec[0]
            this_e = agnlist.spectrum.data["ENERGY"][okspec,:]
            this_s = agnlist.spectrum.data["FLUX"][okspec,:]
            norm = sptools.integ_spectr(this_e,this_s,e_min_below[i],e_max_below[i])
            this_s *= flux_below[i]/norm
            f=interpolate.interp1d(this_e,this_s,bounds_error=False,fill_value=0.0)
            s += f(e)
        print ""

        agnlist = 0

        s /= area_sr

        # Here norm052=-1 means that the spectrum is already normalized per steradian
        simputname = '%s-lim%3.2e' % (filename,fluxlim)
        make_bkgpatch.make_bkgpatch(bkgname,area_def=area_def,area_type=area_type,
                                    norm052=-1,size=100,dir=dir,specset=(simputname,e,s))
    
        print "Done"

    #return e,s


#-------------------------------------------------
def gilli_realisation(outfile,cosmo,area_def,area_type='',zbounds=[0.015,5.],dirtemp="/tmp/",npoints=2000,nproc=1,skipflux=False,fullSpec=False,specBank=['','',''],fixGalNH=-1,namepref='',oversamplesky=False,nhfast=True,impose_number=False):

    """
    Provides a random source list following Gilli et al 2007 modelling.
    * outfile is the name of the output SIMPUT file (disregarded if skipflux=True)
    * cosmo is a dictionnary with cosmology description
    Must contain at least 'h','omega_M_0','omega_lambda_0' and 'omega_k_0'
    * area_def is a [[ramin,ramax],[decmin,decmax]] list or a [ra_cen, dec_cen, radius] list
    * area_type: 'angularbox' for the first case ; 'projcircle' in the second case
    * zbounds are the limiting redshift, e.g. [0.05,5] (note that 0 may cause integration issues)
    * dirtemp: working directory for XSpec to calculate Gilli models
    * npoints: number of energy points for computing spectral models
    * nproc: number of processors for computing models
    * skipflux: set to True to avoid the spectrum computation and return only Lx, z, Nh and Gamma
    * fullSpec: set to True if each source must have its spectrum fully computed
    * specBank:['','',''] name of files for Unabs/Cthin/Cthick spectral banks.
    * fixGalNH: set the galactic NH value
    (set to -1 if you want to get it from the actual RA,DEC source position)
    * namepref: a prefix to put in front of each source name
    * oversamplesky: uses the simulated list to full sky by using the same sources as simulated
    in the area and resampling to full sky.
    * nhfast: if False, uses nh tool, otherwise reads the map

    AUTHOR: Nicolas Clerc (NC)/MPE 2012-2016

    UPDATES:
    - 24.11.2014: impose number of sources via command line? -- NC
    - 09.06.2014: oversample to sky functionnality -- NC
    - 26.06.2013: possibility to prefix the source names (retrocompatible) -- NC
    - 26.11.2012: adds the possibility to define a (RA_CEN, DEC_CEN, radius) geometry instead of a [[RAMIN,RAMAX],[DECMIN,DECMAX]] -- NC
    """

    # -== Positions and area ==-
    area_sky_sr = 4.0*pi
    if(area_type == 'angularbox'):
        ramin = area_def[0][0]*pi/180.
        ramax = area_def[0][1]*pi/180.
        decmin = area_def[1][0]*pi/180.
        decmax = area_def[1][1]*pi/180.
        area_sr = (ramax-ramin)*(cos(pi/2.-decmax)-cos(pi/2.-decmin))
        area = area_sr*(180./pi)**2
    elif(area_type == 'projcircle'):
        ra_cen = area_def[0]
        dec_cen = area_def[1]
        radius = area_def[2]
        area = pi*radius**2
        area_sr = area*(pi/180.)**2.
    else:
        print "Error: must specify area_type"
        sys.exit(1)
    print "... Computing for total area %f deg2 ..." % area

    
    # Some useful parameters
    meangamma = 1.9
    sigmagamma = 0.2
    lbounds =[42,48.] # Valid for all bands...
    QSO_thresh = 10.**(46.)
    
    
    ## -== Set up spectral banks ==-
    bankunabs  = specBank[0]
    bankcthin  = specBank[1]
    bankcthick = specBank[2]
    if(not skipflux):
        if(not fullSpec):
            for f in specBank:
                if(not os.path.exists(f)):
                    print "Bank %s seems to be missing. Check input file names." % f
                    sys.exit(1)
                print "  > Found spectral bank %s" % f



    ## -==  Prepare conversion factors ==-
    ## These tables contain the conversion factors Lsoft/Lband for two bands
    ## which are 2-10 and 10-1000 keV
    ## And for two types of unabsorbed spectra: QSO and Seyfert
    print "... Preparing conversion coefficients ..."
    mingamma = meangamma-4.*sigmagamma
    maxgamma = meangamma+4.*sigmagamma
    ngammas  = 10
    conv_210   = prepare_convfac(bankunabs,mingamma,maxgamma,ngammas,[2.0,10.0])
    conv_101000 = prepare_convfac(bankunabs,mingamma,maxgamma,ngammas,[10.0,1000.0])

    # Interpolators
    fsey210=interpolate.interp1d(conv_210["gamma"],conv_210["sey"])
    fqso210=interpolate.interp1d(conv_210["gamma"],conv_210["qso"])
    fsey101000=interpolate.interp1d(conv_101000["gamma"],conv_101000["sey"])
    fqso101000=interpolate.interp1d(conv_101000["gamma"],conv_101000["qso"])

    
    ## -== Total expected number of sources ==-
    print "... Integrating distributions ..."
    # 1. Unobscured
    print " a. Unobscured"
    gfun= lambda x: lbounds[0]
    hfun= lambda x: lbounds[1]
    unobs,err = integrate.dblquad(unobscured_integrand,zbounds[0],zbounds[1],gfun,hfun,args=(cosmo,),epsrel=1.e-03,epsabs=1.e-03)
    N_unobs = area_sr*unobs

    # 2. Obscured
    ## (formally one should integrate the Cthin and Cthick distributions 
    ## which also depend on gamma. But here, one exploits the fact that lbounds covers almost all the desired range of luminosities and we can drop the gamma dependence.
    #gfun= lambda x: lbounds[0]
    #hfun= lambda x: lbounds[1]
    #obsc,err = integrate.dblquad(obscured_integrand,zbounds[0],zbounds[1],gfun,hfun,args=(cosmo,))
    #N_obsc = area_sr*obsc*2 # Factor 2 to account for Cthin+Cthick
    #N_Cthin = N_obsc/2
    #N_Cthick = N_obsc/2
    
    # Now, we want to use the PDF of gamma which is integrated to simplify the job
    ngvalues = 10
    gvalues = np.linspace(meangamma-3.*sigmagamma,meangamma+3.*sigmagamma,ngvalues+1)
    gmin = gvalues[0:-1]
    gmax = gvalues[1:]
    gcen = 0.5*(gmin+gmax)
    weights = calc_pi(gmin,gmax,meangamma,sigmagamma)
   
    # 2. Cthin
    print " b. CThin"
    lbounds_CThin = [lbounds[0],lbounds[1]] # L[2-10] # ---> Non, plus maintenant : 0.5-2
    gfun= lambda x: lbounds_CThin[0]
    hfun= lambda x: lbounds_CThin[1]
    N_Cthin = 0.
    for k in range(ngvalues):
        print "  Bin %2i/%2i gamma in [%f;%f]" % (k+1,ngvalues,gmin[k],gmax[k])
        #convsey = fsey210(gcen[k])
        convsey = 1.
        #convqso = fqso210(gcen[k])
        convqso = 1.
        fn = lambda x,y: obscured_integrand_gamma(x,y,cosmo,convsey,convqso,QSO_thresh)
        cthin,err = integrate.dblquad(fn,zbounds[0],zbounds[1],gfun,hfun,epsabs=1.e-03,epsrel=1.e-03)
        N_Cthin += weights[k]*cthin
    N_Cthin *= area_sr
       
    # 3. Cthick
    print " c. CThick"
    lbounds_CThick = [lbounds[0],lbounds[1]] # L[10-1000] # ---> Non, plus maintenant: 0.5-2
    gfun= lambda x: lbounds_CThick[0]
    hfun= lambda x: lbounds_CThick[1]
    N_Cthick = 0.
    for k in range(ngvalues):
        print "  Bin %2i/%2i gamma in [%f;%f]" % (k+1,ngvalues,gmin[k],gmax[k])
       #convsey = fsey101000(gcen[k])
        convsey = 1.
       #convqso = fqso101000(gcen[k])
        convqso = 1.
        fn = lambda x,y: obscured_integrand_gamma(x,y,cosmo,convsey,convqso,QSO_thresh)
        cthick,err = integrate.dblquad(fn,zbounds[0],zbounds[1],gfun,hfun,epsabs=1.e-03,epsrel=1.e-03)
        N_Cthick += weights[k]*cthick
    N_Cthick *= area_sr
    
    print " Done"
   

    ## -== Pick a random number of sources ==-

    if(impose_number):
        print "***********************"
        Ntot = N_unobs+N_Cthin+N_Cthick
        print "Expected = %f" % Ntot
        print "***********************"
        Nsources = raw_input("==> Enter number of sources:")
        Nsources = float(Nsources)
        Nsources_unobs = int(floor((N_unobs/Ntot)*Nsources))
        Nsources_cthin = int(floor((N_Cthin/Ntot)*Nsources))
        Nsources_cthick = int(floor((N_Cthick/Ntot)*Nsources))
        Nsources_cthick += Ntot - (Nsources_unobs+Nsources_cthin+Nsources_cthick)
    else:
        Nsources_unobs = np.random.poisson(N_unobs)
        Nsources_cthin = np.random.poisson(N_Cthin)
        Nsources_cthick = np.random.poisson(N_Cthick)
        Nsources = Nsources_unobs + Nsources_cthin+Nsources_cthick

    print "... Simulating %i sources ..." % Nsources
    print "    ( %i Unobscured, %i Cthin, %i Cthick)" % (Nsources_unobs,Nsources_cthin, Nsources_cthick)
    
    
   ## -== Random slope index + 4-sigma threshold ==-
    all_gamma = np.random.normal(loc=meangamma,scale=sigmagamma,size=Nsources)
    all_gamma[all_gamma < meangamma-4.*sigmagamma] = meangamma-4.*sigmagamma
    all_gamma[all_gamma > meangamma+4.*sigmagamma] = meangamma+4.*sigmagamma
    
    print "... Sampling distributions ..."
    
   ## --------------------------------== (1) Unobscured sample ==-
    unobs_lx = np.zeros(Nsources_unobs)
    unobs_z  = np.zeros(Nsources_unobs)
    unobs_nh= np.zeros(Nsources_unobs)-1
    unobs_gamma = all_gamma[0:Nsources_unobs]

    # Rejection sampling
    nregistered = 0
    
    # Find maximum of probability distribution (expected to be at high z)
    fn = lambda x,y: unobscured_integrand(x,y,cosmo)
    maxi_f = rough_maximum(fn,lbounds,zbounds)
    g_value = 1./( (lbounds[1]-lbounds[0])*(zbounds[1]-zbounds[0]) ) ## Uniform
    c_coeff = 1.1*maxi_f/g_value # 10% security margin
   
    while(nregistered < Nsources_unobs):
      ## Uniform distrib (logLx,z)
        logLx = lbounds[0]+(lbounds[1]-lbounds[0])*np.random.random()
        z     = zbounds[0]+(zbounds[1]-zbounds[0])*np.random.random()
        unif  = np.random.random()
        if(c_coeff*g_value*unif < unobscured_integrand(logLx,z,cosmo)):
            unobs_lx[nregistered] = 10.**logLx
            unobs_z[nregistered] = z
            nregistered += 1
            print "  Unabs: %3.0f " % (100.*nregistered/Nsources_unobs),"percent complete    \r",
    print ""


    ## ------------------------------------== (2) CThin sample ==-
    cthin_lx052 = np.zeros(Nsources_cthin)
    cthin_z  = np.zeros(Nsources_cthin)
    cthin_gamma = all_gamma[Nsources_unobs:Nsources_unobs+Nsources_cthin]

    # Find the maximum for different values of gamma
    mingamma = meangamma-4.*sigmagamma
    maxgamma = meangamma+4.*sigmagamma
    ngammas  = 15
    gammaval = np.linspace(mingamma,maxgamma,ngammas)
    maxi_f_gamma = np.zeros(ngammas)
    for g in range(ngammas):
        #convsey = fsey210(gammaval[g])
        convsey = 1.
        #convqso = fqso210(gammaval[g])
        convqso = 1.
        fn = lambda x,y: obscured_integrand_gamma(x,y,cosmo,convsey,convqso,QSO_thresh)
        maxi_f_gamma[g] = rough_maximum(fn,lbounds_CThin,zbounds)
    fn = interpolate.interp1d(gammaval,maxi_f_gamma)
    
    nregistered = 0
    for s in range(Nsources_cthin):
        src_gamma = cthin_gamma[s]
        #convsey = fsey210(src_gamma)
        convsey = 1.
      #convqso = fqso210(src_gamma)
        convqso = 1.
      # Find maximum of probability distribution
        maxi_f = fn(src_gamma)
        g_value = 1./( (lbounds_CThin[1]-lbounds_CThin[0])*(zbounds[1]-zbounds[0]) ) ## Uniform
        c_coeff = 1.2*maxi_f/g_value # 20% security margin
        
        registered = False
        while(not registered):
            ## Uniform distrib (logLx,z)
            logLx = lbounds_CThin[0]+(lbounds_CThin[1]-lbounds_CThin[0])*np.random.random()
            z = zbounds[0] + (zbounds[1]-zbounds[0])*np.random.random()
            unif = np.random.random()           
            trial = obscured_integrand_gamma(logLx,z,cosmo,convsey,convqso,QSO_thresh)
            if(c_coeff*g_value*unif < trial):
                cthin_lx052[s] = 10.**logLx
                cthin_z[s] = z
                registered = True
                nregistered += 1
                print "  Cthin: %3.0f " % (100.*nregistered/Nsources_cthin),"percent complete    \r",

    print ""

    # Assign Nh values randomly
    # Invert cumulative distribution of Nh then draw from it
    unif = np.random.random(Nsources_cthin)
    lognhmin=np.array([21.,22.,23.])
    lognhmax=np.array([22.,23.,24.])
    fracnh = np.array([0.0476821,0.3523179,0.6])    
    
    cthin_nh = map(lambda y: invert_nhcumul(y,lognhmin,lognhmax,fracnh),unif)
    cthin_nh = np.array(cthin_nh)[:,0]
    
    
    ## ------------------------------== (3) CThick sample ==-
    cthick_lx052 = np.zeros(Nsources_cthick)
    cthick_z  = np.zeros(Nsources_cthick)
    cthick_gamma = all_gamma[Nsources_unobs+Nsources_cthin:Nsources_unobs+Nsources_cthin+Nsources_cthick]


    # Find the maximum for different values of gamma
    mingamma = meangamma-4.*sigmagamma
    maxgamma = meangamma+4.*sigmagamma
    ngammas  = 15
    gammaval = np.linspace(mingamma,maxgamma,ngammas)
    maxi_f_gamma = np.zeros(ngammas)
    for g in range(ngammas):
        #convsey = fsey101000(gammaval[g])
        convsey = 1.
        #convqso = fqso101000(gammaval[g])
        convqso = 1.
        fn = lambda x,y: obscured_integrand_gamma(x,y,cosmo,convsey,convqso,QSO_thresh)
        maxi_f_gamma[g] = rough_maximum(fn,lbounds_CThick,zbounds)
    fn = interpolate.interp1d(gammaval,maxi_f_gamma)
    
    nregistered = 0
    for s in range(Nsources_cthick):
        src_gamma = cthick_gamma[s]
        #convsey = fsey101000(src_gamma)
        convsey = 1.
        #convqso = fqso101000(src_gamma)
        convqso = 1.
        # Find maximum of probability distribution
        maxi_f = fn(src_gamma)
        g_value = 1./( (lbounds_CThick[1]-lbounds_CThick[0])*(zbounds[1]-zbounds[0]) ) ## Uniform
        c_coeff = 1.2*maxi_f/g_value # 20% security margin
        
        registered = False
        while(not registered):
        ## Uniform distrib (logLx,z)
            logLx = lbounds_CThick[0]+(lbounds_CThick[1]-lbounds_CThick[0])*np.random.random()
            z = zbounds[0] + (zbounds[1]-zbounds[0])*np.random.random()
            unif = np.random.random()                   
            trial = obscured_integrand_gamma(logLx,z,cosmo,convsey,convqso,QSO_thresh)
            if(c_coeff*g_value*unif < trial ):
                cthick_lx052[s] = 10.**logLx
                cthick_z[s] = z
                registered = True        
                nregistered += 1
                print "  Cthick: %3.0f " % (100.*nregistered/Nsources_cthick),"percent complete    \r",

    print ""

    # Assign Nh values randomly
    # Invert cumulative distribution of Nh then draw from it
    unif = np.random.random(Nsources_cthick)
    lognhmin=np.array([24.,25.])
    lognhmax=np.array([25.,26.])
    fracnh = np.array([0.5,0.5])    

    cthick_nh = map(lambda y: invert_nhcumul(y,lognhmin,lognhmax,fracnh),unif)
    cthick_nh = np.array(cthick_nh)[:,0]

    ####
    #print " FIXING g=1.9 for NH>24, NH<25"
    #cthick_gamma[(cthick_nh >= 24.)*(cthick_nh < 25.)] = 1.9
    ###


    ## At this stage, one has:
    ## 1) A sample of LX[0.5,2],z,gamma values for unabsorbed sources
    ## 2) A sample of LX[2,10] ,z,gamma,Nh values for Compton Thin sources
    ## 3) A sample of LX[10,1000],z,gamma,Nh values for Compton Thick sources


    # -== Concatenate distributions ==-
    all_lx = np.concatenate((unobs_lx,cthin_lx052,cthick_lx052))
    all_z  = np.concatenate((unobs_z,cthin_z,cthick_z))
    all_lognh = np.concatenate((unobs_nh,cthin_nh,cthick_nh))


    ## Now we have Nsources with Lx,Nh,z and gamma randomly distributed
    if(skipflux):
        print "Returning only Lx [Warning//Different E ranges], z, logNH and gamma..."
        return all_lx,all_z,all_lognh,all_gamma



    ## ---------------------------== Compute source spectra ==-
    print "... Computing source spectral models ..."
    
    if(fullSpec):
        # Option 1.:re-compute each model // Parallel jobs
        # Unabsorbed spectra [unnormalized]
        #print "... 1) Unabsorbed ..."
        #idint = map(str,range(Nsources_unobs))
        #idstr = ["spec_unabs_"+u+"_" for u in idint]
        #res = Parallel(n_jobs=nproc)(delayed(gi.mk_gillimodel)(dirtemp,unobs_gamma[i],10.**unobs_nh[i],unobs_z[i],
        #                                                       unobs_lx[i],bandlum=[0.5,2.],npoints=npoints,verbose=1,
        #                                                       addstring=idstr[i]) for i in range(Nsources_unobs))
        #ener_unobs,spec_unobs = zip(*res)
        #
        ## Compton Thin [unnormalized]
        #print "... 2) Compton-Thin ..."
        #idint = map(str,range(Nsources_cthin))
        #idstr = ["spec_cthin_"+u+"_" for u in idint]
        #res = Parallel(n_jobs=nproc)(delayed(gi.mk_gillimodel)(dirtemp,cthin_gamma[i],10.**cthin_nh[i],cthin_z[i],
        #                                                       cthin_lx210[i],bandlum=[2.,10.],npoints=npoints,verbose=1,
        #                                                       addstring=idstr[i]) for i in range(Nsources_cthin))
        #ener_cthin,spec_cthin = zip(*res)

        ## Compton Thick [unnormalized]
        #print "... 3) Compton-Thick ..."
        #idint = map(str,range(Nsources_cthick))
        #idstr = ["spec_cthick_"+u+"_" for u in idint]
        #res = Parallel(n_jobs=nproc)(delayed(gi.mk_gillimodel)(dirtemp,cthick_gamma[i],10.**cthick_nh[i],cthick_z[i],
        #                                                       cthick_lx101000[i],bandlum=[10.,1000.],npoints=npoints,verbose=1,
        #                                                       addstring=idstr[i]) for i in range(Nsources_cthick))
        #ener_cthick,spec_cthick = zip(*res)
        print "Not implemented..."
        sys.exit(1)

    else:
        # Option 2: read from a table of existing files // Faster
        ener_unobs,spec_unobs,type_unobs,norm052_unobs,norm210_unobs,norm101000_unobs = gi.read_specbank(bankunabs,unobs_gamma,10.**unobs_nh,unobs_z,unobs_lx,bandlum=[0.5,2.],QSO_thresh=QSO_thresh)
        nsey = type_unobs.count('sey')
        nqso = type_unobs.count('qso')
        print "  [Unabs: %i Sey and %i QSO]" % (nsey,nqso)
        
        ener_cthin,spec_cthin,type_cthin,norm052_cthin,norm210_cthin,norm101000_cthin = gi.read_specbank(bankcthin,cthin_gamma,10.**cthin_nh,cthin_z,cthin_lx052,bandlum=[0.5,2.],QSO_thresh=QSO_thresh)
        nsey = type_cthin.count('sey')
        nqso = type_cthin.count('qso')
        print "  [Cthin: %i Sey and %i QSO]" % (nsey,nqso)

        ener_cthick,spec_cthick,type_cthick,norm052_cthick,norm210_cthick,norm101000_cthick = gi.read_specbank(bankcthick,cthick_gamma,10.**cthick_nh,cthick_z,cthick_lx052,bandlum=[0.5,2.],QSO_thresh=QSO_thresh)
        nsey = type_cthick.count('sey')
        nqso = type_cthick.count('qso')
        print "  [Cthick: %i Sey and %i QSO]" % (nsey,nqso)



    ## -== Do we resample to full sky? ==-
    if(oversamplesky):
        print "Full-sky oversampling activated!"
        areafac = area_sky_sr/area_sr
        Nsources_unobs_orig = Nsources_unobs
        Nsources_cthin_orig = Nsources_cthin
        Nsources_cthick_orig = Nsources_cthick
        Nsources_unobs = np.random.poisson(N_unobs*areafac)
        Nsources_cthin = np.random.poisson(N_Cthin*areafac)
        Nsources_cthick = np.random.poisson(N_Cthick*areafac)
        
        Nsources = Nsources_unobs + Nsources_cthin+Nsources_cthick
        print "... Simulating %i sources ..." % Nsources
        print "    ( %i Unobscured, %i Cthin, %i Cthick)" % (Nsources_unobs,Nsources_cthin, Nsources_cthick)
       # Now modify the type of geometry
        area_def  = [[0.,360.],[-90.,90]]
        area_type = 'angularbox'

        # Oversample the spectral indices!
        # Indexation is j=random
        unobs_index = np.random.randint(0,Nsources_unobs_orig,size=Nsources_unobs)
        cthin_index = np.random.randint(0,Nsources_cthin_orig,size=Nsources_cthin)
        cthick_index = np.random.randint(0,Nsources_cthick_orig,size=Nsources_cthick)
    else:
       # No sampling, indexation is j=i
        unobs_index = range(Nsources_unobs)
        cthin_index = range(Nsources_cthin)
        cthick_index = range(Nsources_cthick)


    ## -== Generate positions ==-
    print "... Generate random positions ..."
    all_ra,all_dec = gen_positions(Nsources,area_def,geomtype=area_type)
    

    ## -== Galactic absorption ==-
    print "... Apply galactic absorption ..."
    if(nhfast):
        if(fixGalNH <0):
            nhgal = sptools.get_nh_fast(all_ra,all_dec)
        else:
            nhgal = np.array([fixGalNH]*Nsources)
    else:
        nhgal = np.zeros(Nsources)
        for i in range(Nsources):
            if(fixGalNH <0):
                print "  Done: %3.0f " % (100.*i/Nsources),"  \r",
                nhgal[i] = sptools.get_nh(all_ra[i],all_dec[i])
            else:
                nhgal[i] = fixGalNH
        print ""
            
    ## -== Calculate unabsorbed and absorbed fluxes ==-
    print "... Calculate fluxes ..."
    all_f052 = np.zeros(Nsources)
    all_f210 = np.zeros(Nsources)
    all_f510 = np.zeros(Nsources)
    all_f052_abs = np.zeros(Nsources)
    all_lxsoft = np.zeros(Nsources)
    Mpc2cm = 3.08568025e24
    all_spec_abs = []
    

    # ~ Unobscured ~
    for k in range(Nsources_unobs):
        i = unobs_index[k]
        print " Unabsorbed source %i/%i spectrum processing   \r" %(i+1,Nsources_unobs),
        energy = ener_unobs[i]
        spectrum = spec_unobs[i]
        
        # Rest-frame 0.5-2 spectral integration
        #factor = sptools.integ_spectr(energy,spectrum,0.5/(1+unobs_z[i]),2./(1+unobs_z[i]))
        scalingfactor = unobs_lx[i]/norm052_unobs[i]
       
       # Observer-frame 0.5-2/2-10/5-10 spectral integration
        spec052 = sptools.integ_spectr(energy,spectrum,0.5,2.0)
        spec210 = sptools.integ_spectr(energy,spectrum,2.0,10.0)
        spec510 = sptools.integ_spectr(energy,spectrum,5.0,10.0)
       # Observer-frame 0.5-2/2-10/5-10 luminosity
        lum052 = spec052*scalingfactor
        lum210 = spec210*scalingfactor
        lum510 = spec510*scalingfactor
       # Observer-frame 0.5-2/2-10/5-10 flux
        lumdist = cd.luminosity_distance(unobs_z[i], **cosmo)*Mpc2cm
        denomin = (4.*pi*lumdist**2)
        all_f052[0+i] = lum052/denomin
        all_f210[0+i] = lum210/denomin
        all_f510[0+i] = lum510/denomin
        all_lxsoft[0+i] = unobs_lx[i]
       # Apply galactic absorption
        newspec = (spectrum)*sptools.wabs(nhgal[0+i],energy)
        spec052_abs = sptools.integ_spectr(energy,newspec,0.5,2.0)
        all_f052_abs[0+i] = (spec052_abs*scalingfactor)/denomin
        all_spec_abs.append(newspec)

    print ""

    # ~ Cthin ~
    for k in range(Nsources_cthin):
        i = cthin_index[k]
        print " Compton-Thin source %i/%i spectrum processing   \r" %(i+1,Nsources_cthin),
        energy = ener_cthin[i]
        spectrum = spec_cthin[i]
       
        # Rest-frame 2-10 spectral integration
        #factor = sptools.integ_spectr(energy,spectrum,2.0/(1+cthin_z[i]),10./(1+cthin_z[i]))
        scalingfactor = cthin_lx052[i]/norm052_cthin[i]

        # Observer-frame 0.5-2/2-10/5-10 spectral integration
        spec052 = sptools.integ_spectr(energy,spectrum,0.5,2.0)
        spec210 = sptools.integ_spectr(energy,spectrum,2.0,10.0)
        spec510 = sptools.integ_spectr(energy,spectrum,5.0,10.0)
        # Observer-frame 0.5-2/2-10/5-10 luminosity
        lum052 = spec052*scalingfactor
        lum210 = spec210*scalingfactor
        lum510 = spec510*scalingfactor
        # Observer-frame 0.5-2/2-10/5-10 flux
        lumdist = cd.luminosity_distance(cthin_z[i], **cosmo)*Mpc2cm
        denomin = (4*pi*lumdist**2)
        all_f052[Nsources_unobs+i] = lum052/denomin
        all_f210[Nsources_unobs+i] = lum210/denomin
        all_f510[Nsources_unobs+i] = lum510/denomin
        # Store rest-frame 0.5-2 luminosity
        rf052 = norm052_cthin[i]
        all_lxsoft[Nsources_unobs+i] = rf052*scalingfactor
        # Apply galactic absorption
        newspec = (spectrum)*sptools.wabs(nhgal[Nsources_unobs+i],energy)
        spec052_abs = sptools.integ_spectr(energy,newspec,0.5,2.0)
        all_f052_abs[Nsources_unobs+i] = (spec052_abs*scalingfactor)/denomin
        all_spec_abs.append(newspec)

    print ""

    # ~ Cthick ~
    for k in range(Nsources_cthick):
        i = cthick_index[k]
        print " Compton-Thick source %i/%i spectrum processing   \r" %(i+1,Nsources_cthick),
        energy = ener_cthick[i]
        spectrum = spec_cthick[i]

        # Rest-frame 10-1000 spectral integration
        #factor = sptools.integ_spectr(energy,spectrum,10.0/(1+cthick_z[i]),1000./(1+cthick_z[i]))
        scalingfactor = cthick_lx052[i]/norm052_cthick[i]
       
        # Observer-frame 0.5-2/2-10/5-10 spectral integration
        spec052 = sptools.integ_spectr(energy,spectrum,0.5,2.0)
        spec210 = sptools.integ_spectr(energy,spectrum,2.0,10.0)
        spec510 = sptools.integ_spectr(energy,spectrum,5.0,10.0)
        # Observer-frame 0.5-2/2-10/5-10 luminosity
        lum052 = spec052*scalingfactor
        lum210 = spec210*scalingfactor
        lum510 = spec510*scalingfactor
        # Observer-frame 0.5-2/2-10/5-10 flux
        lumdist = cd.luminosity_distance(cthick_z[i], **cosmo)*Mpc2cm
        denomin = (4*pi*lumdist**2)
        all_f052[Nsources_unobs+Nsources_cthin+i] = lum052/denomin
        all_f210[Nsources_unobs+Nsources_cthin+i] = lum210/denomin
        all_f510[Nsources_unobs+Nsources_cthin+i] = lum510/denomin
        # Store rest-frame 0.5-2 luminosity
        rf052 = norm052_cthick[i]
        all_lxsoft[Nsources_unobs+Nsources_cthin+i] = rf052*scalingfactor
        # Apply galactic absorption
        newspec = (spectrum)*sptools.wabs(nhgal[Nsources_unobs+Nsources_cthin+i],energy)
        spec052_abs = sptools.integ_spectr(energy,newspec,0.5,2.0)
        all_f052_abs[Nsources_unobs+Nsources_cthin+i] = (spec052_abs*scalingfactor)/denomin
        all_spec_abs.append(newspec)


    print ""

    all_ener = ener_unobs+ener_cthin+ener_cthick
    all_spec = spec_unobs+spec_cthin+spec_cthick
    ener_unobs = 0
    ener_cthin = 0
    ener_cthick= 0
    spec_unobs = 0
    spec_cthin = 0
    spec_cthick= 0

    ## -== Give names ==-
    all_names = np.array(['']*Nsources,dtype='|S256')
    for i in range(Nsources):  
        # - Find a name
        if(i < Nsources_unobs):
            src_name = '%sunobs_%i' % (namepref,i)
        elif( (Nsources_unobs <= i) and (i < Nsources_unobs+Nsources_cthin)):
            src_name = '%scthin_%i' % (namepref,i)
        else:
            src_name = '%scthick_%i' % (namepref,i)
        all_names[i] = src_name


    ## -== Make the simput catalogue + spectrum ==-
    if(outfile != ''):
        print "... Writing SIMPUT catalogue ..."
        splength = np.size(all_ener[0])
        mysim = simput.simputobj(['catalogue','spectrum'],cat_columns=['SRC_NAME'],spec_columns=['NAME'],speclength=splength)
       
        # ~ Simput Spectrum ~
        print "  a) Registering spectra"
        specid = []
        spname = []
        for i in range(Nsources):
            spid = '%sspecsrc%i' % (namepref,i)
            specid.append("[SPECTRUM,1][NAME=='%s']" % spid)
            spname.append(spid)
        mysim.spectrum.data["NAME"] = np.array(spname)
        mysim.spectrum.data["ENERGY"] = all_ener
        mysim.spectrum.data["FLUX"] = all_spec_abs
        mysim.spectrum.nspectra = Nsources


        # ~ Simput Catalogue ~
        print "  b) Registering sources"
        #for i in range(Nsources):
            #print "    %i/%i complete   \r" %(i+1,Nsources),
            # - Build source
            #nouvelle = {"SRC_ID":i,"SRC_NAME":all_names[i],
            #            "RA":all_ra[i],"DEC":all_dec[i],
            #            "E_MIN":0.5,"E_MAX":2.0,"FLUX":all_f052_abs[i],
            #            "SPECTRUM":"'"+specid[i]+"'","IMAGE":'NULL',
            #            "TIMING":'NULL'}
            # - Add source
            #mysim.catalogue.addSource(nouvelle)
        #print ""
        mysim.catalogue.data["SRC_ID"] = np.arange(Nsources)
        mysim.catalogue.data["SRC_NAME"] = all_names
        mysim.catalogue.data["RA"] = all_ra
        mysim.catalogue.data["DEC"] = all_dec
        mysim.catalogue.data["E_MIN"] = np.zeros(Nsources)+0.5
        mysim.catalogue.data["E_MAX"] = np.zeros(Nsources)+2.0
        mysim.catalogue.data["FLUX"] = all_f052_abs
        mysim.catalogue.data["SPECTRUM"] = np.array(specid) # np.array(["'"+x+"'" for x in specid])
        mysim.catalogue.data["IMAGE"]  = np.array(['NULL']*Nsources)
        mysim.catalogue.data["TIMING"] = np.array(['NULL']*Nsources)


        print "  c) Writing file"
        mysim.writeto(outfile)

    ## -== Format result ==-
    resdic = {"simputid":all_names,"lum":all_lx,"lum052":all_lxsoft,"z":all_z,"logNh":all_lognh,
              "gamma":all_gamma,"f052":all_f052,"f210":all_f210,"f510":all_f510,
              "f052_abs":all_f052_abs}
    

    return resdic









# produce single double power-law random distribution
def readcatFROMfits(fitsfile="", RAKEY='ra', DECKEY="dec", FLUXKEY="flux052"):
    import pyfits
    
    hdu = pyfits.open(fitsfile)
    ra=hdu[1].data[RAKEY]
    dec=hdu[1].data[DECKEY]
    flux=hdu[1].data[FLUXKEY]

    m = np.isfinite(flux)
    ra=ra[m]
    dec=dec[m]
    flux=flux[m]
   
                   
    m = flux > 0.0
    
    ra=ra[m]
    dec=dec[m]
    flux=flux[m]
    hdu.close()

    return {'RA':ra, 'DEC':dec, 'F': flux};

    
def dnds_dpl(beta1=-1.58, beta2=-2.5, norm=1.51e16, lgf14_break=-0.04, lgf14=-1):

    # dpl logN-logS as in Georgakakis et al. 2008
    # dN/dS = norm * S14^beta1 if S14<S14_break 
    # dN/dS = kprime * S14^beta2 if S14>S14_break 
    # krpime = S14_break**beta1 /  S14_break**beta2
    
    
    kprime = norm * 10**(lgf14_break * (beta1 - beta2) )
    #print lgf14
    #print kprime
    a1 = np.log10(norm) + beta1 * lgf14
    a2 = np.log10(kprime) + beta2 * lgf14

    
    
    if(isinstance(lgf14, np.ndarray)):
        mask = lgf14 > lgf14_break
        dnds[mask] = a2[mask]
 
    else:
        dnds = a1
        if(lgf14 > lgf14_break):
            dnds = a2

    return dnds

def random_dpl(beta1=-1.58, beta2=-2.5, norm=1.51e16, lgf14_break=-0.04, lgf14=-1, area=100):

    # dpl logN-logS as in Georgakakis et al. 2008
    # dN/dS = norm * S14^beta1 if S14<S14_break 
    # dN/dS = kprime * S14^beta2 if S14>S14_break 
    # krpime = S14_break**beta1 /  S14_break**beta2
    
  
    kprime = norm * 10**(lgf14_break * (beta1 - beta2) )

    lgf14c = np.arange(lgf14+4,lgf14-0.01, -0.01)

    
    if(lgf14 < lgf14_break):
        a1 = 10**( (1+beta1) * lgf14_break )  - 10**( (1+beta1) * lgf14) 
        a2 = -10**( (1+beta2) * lgf14_break )
        total = norm * 1e-14 / (1+beta1) * a1 + kprime * 1e-14 / (1+beta2) * a2
    else:
        a2 = -10**( (1+beta2) * lgf14 )
        total = kprime * 1e-14 / (1+beta2) * a2


    #print total, area, kprime,norm, beta1, beta2,lgf14_break,lgf14
    
    a1 = np.power(10,  (1+beta1) * lgf14_break)  - np.power(10, (1+beta1) * lgf14c) 
    a2 = -1.0 * np.power(10, (1+beta2) * lgf14_break )
    a3 = -1.0 * np.power(10, (1+beta2) * lgf14c )
    
    mask = lgf14c < lgf14_break
    cum = kprime * 1e-14 / (1+beta2) * a3
    cum[mask] = norm * 1e-14 / (1+beta1) * a1[mask] + kprime * 1e-14 / (1+beta2) * a2


    random  = np.random.random_sample(int(total*area))

    #print random.size
    
    s = np.interp(random, cum/total, lgf14c)

    #print 10**s * 1e-14
    #print s
    
    return {'F':s};


def popCAT_dpl(paramDICT={'beta1':-1.58, 'beta2':-2.5, 'norm':1.51e16, 'lgf14_break':-0.04, 'lgf14_min':-1, 'area':100.0}, area_def='',area_type=''):
    #print "INS:", paramDICT['beta1'], paramDICT['beta2'], paramDICT['norm'], paramDICT['lgf14_break'], paramDICT['lgf14_min'], paramDICT['area']

    fluxDICT = random_dpl(beta1=paramDICT['beta1'], beta2=paramDICT['beta2'], norm=paramDICT['norm'], lgf14_break=paramDICT['lgf14_break'], lgf14=paramDICT['lgf14_min'], area=paramDICT['area'])
    Nsources = fluxDICT['F'].size

    #print "HERE:",Nsources
    #print "HERE:",fluxDICT['F']
    fluxDICT['F'] = 10**(fluxDICT['F']-14)
    all_ra,all_dec = gen_positions(Nsources,area_def=area_def,geomtype=area_type)
    all_names = popCAT_giveNames(Nsources=Nsources, namepref=paramDICT['namepref'])
    return {'NAME': all_names, 'RA': all_ra, 'DEC': all_dec, 'F': fluxDICT['F']};


def popCAT_ranPOS_fixedFlux(paramDICT={'Nsources':100, 'namepref':''}, area_def='',area_type=''):
    all_ra,all_dec = gen_positions(Nsources=paramDICT['Nsources'],area_def=area_def,geomtype=area_type)
    Nsources=all_ra.size
    all_flux = np.zeros(Nsources)
    all_flux.fill(paramDICT['flux'])
    all_names = popCAT_giveNames(Nsources=Nsources, namepref=paramDICT['namepref'])
    return {'NAME': all_names, 'RA': all_ra, 'DEC': all_dec, 'F': all_flux};

def popCAT_gridPOS_fixedFlux(paramDICT={'namepref':'SRC', 'doffset':10.0/60.0}, area_def=[],area_type=[]):
    all_ra,all_dec = gen_GRIDpositions(area_def=area_def,geomtype=area_type, doffset=paramDICT['doffset'])
    Nsources=all_ra.size
    all_flux = np.zeros(Nsources)
    all_flux.fill(paramDICT['flux'])
    all_names = popCAT_giveNames(Nsources=Nsources, namepref=paramDICT['namepref'])
    return {'NAME': all_names, 'RA': all_ra, 'DEC': all_dec, 'F': all_flux};


def popCAT_readFITS(paramDICT={"fitsfile":"", 'RAKEY':'ra', 'DECKEY':'dec', 'FLUXKEY':'f052', 'namepref':'SRC'}):
    fluxDICT = readcatFROMfits(fitsfile=paramDICT["fitsfile"], RAKEY=paramDICT['RAKEY'], DECKEY=paramDICT['DECKEY'], FLUXKEY=paramDICT['FLUXKEY'])
    Nsources = fluxDICT['F'].size
    all_ra = fluxDICT['RA']
    all_dec = fluxDICT['DEC']
    all_flux = fluxDICT['F']
    all_names = popCAT_giveNames(Nsources=Nsources, namepref=paramDICT['namepref'])
    return {'NAME': all_names, 'RA': fluxDICT['RA'], 'DEC': fluxDICT['DEC'], 'F': fluxDICT['F']};

def popCAT_giveNames(Nsources, namepref):
    all_names = np.array(['']*Nsources,dtype='|S256')
    #all_flux = np.zeros(Nsources)
    for i in range(Nsources):  
        src_name = '%sobs_%i' % (namepref,i)
        all_names[i] = src_name
    return all_names;



#-------------------------------------------------
def age_realisation(outfile='',area_def='',area_type='',paramDICT={}, specBank=[''], Gamma=1.4, band=[0.5,2.0]):

    """
    Modified version of Nicolas' gili_realisation, 
    simply produces a simput list of sources with fixed flux and PL with fixed gamma
    this is for calibration purposes

    Provides a random source list following Gilli et al 2007 modelling.
    * outfile is the name of the output SIMPUT file (disregarded if skipflux=True)
    * area_def is a [[ramin,ramax],[decmin,decmax]] list or a [ra_cen, dec_cen, radius] list
    * area_type: 'angularbox' for the first case ; 'projcircle' in the second case
    * specBank:[''] name of files for PL spectral bank
    * namepref: a prefix to put in front of each source name
    * flux: flux is fixed, to do produce logN-logS fluxes
    * energy interval within which flux is estimated. 

    AUTHOR: Nicolas Clerc (NC)/MPE 2012-2016

    UPDATES:
    - 24.11.2014: impose number of sources via command line? -- NC
    - 09.06.2014: oversample to sky functionnality -- NC
    - 26.06.2013: possibility to prefix the source names (retrocompatible) -- NC
    - 26.11.2012: adds the possibility to define a (RA_CEN, DEC_CEN, radius) geometry instead of a [[RAMIN,RAMAX],[DECMIN,DECMAX]] -- NC
    """

    # -== Positions and area ==-
    area_sky_sr = 4.0*pi
    if(area_type == 'angularbox'):
        ramin = area_def[0][0]*pi/180.
        ramax = area_def[0][1]*pi/180.
        decmin = area_def[1][0]*pi/180.
        decmax = area_def[1][1]*pi/180.
        area_sr = (ramax-ramin)*(cos(pi/2.-decmax)-cos(pi/2.-decmin))
        area = area_sr*(180./pi)**2
    elif(area_type == 'projcircle'):
        ra_cen = area_def[0]
        dec_cen = area_def[1]
        radius = area_def[2]
        area = pi*radius**2
        area_sr = area*(pi/180.)**2.
    else:
        print "Error: must specify area_type"
        sys.exit(1)
    print "... Computing for total area %f deg2 ..." % area, ra_cen, dec_cen

    
    # this is the spectral bank
    # it also checks if files are there
    bankPL  = specBank[0]    
    for f in specBank:
        if(not os.path.exists(f)):
            print "Bank %s seems to be missing. Check input file names." % f
            sys.exit(1)
        print "  > Found spectral bank %s" % f
        

    # Option 2: read from a table of existing files // Faster
    # AGE: HERE read_specbankPL....
    energ,spec,good,norm052,norm210,norm101000 = gi.read_specbankPL(bankPL,np.array([Gamma]),np.array([0.1]),np.array([1e44]),bandlum=[0.5,2.])

    #print energ
    #print spec

    if (paramDICT['CHOOSE'] == 1):
        paramDICT['area']=area
        mySRC = popCAT_dpl(paramDICT, area_def, area_type)        
    if (paramDICT['CHOOSE'] == 2):
        mySRC = popCAT_ranPOS_fixedFlux(paramDICT, area_def, area_type)
    if (paramDICT['CHOOSE'] == 3):        
        mySRC =  popCAT_readFITS(paramDICT)
    if (paramDICT['CHOOSE'] == 4):        
        mySRC =  popCAT_gridPOS_fixedFlux(paramDICT, area_def,area_type)
        
#    ## -== Generate positions ==-
#    ## AGE : HERE for positions
#    if(grid_positions):
#        print "... Generate grid positions ..."
#        all_ra,all_dec = gen_GRIDpositions(area_def=area_def,geomtype=area_type, doffset=10./60.)
#        Nsources=all_ra.size
#        print all_ra[0:10]
#    else:
#        print "... Generate random positions ..."
#        all_ra,all_dec = gen_positions(Nsources,area_def,geomtype=area_type)
    
        # this is the number of sources that will be simulated
    all_ra=mySRC['RA']
    all_dec = mySRC['DEC']
    #all_flux = np.power(10,mySRC['F'])*1e-14
    all_flux = mySRC['F']
    all_names = mySRC['NAME']
    Nsources = mySRC['RA'].size
    Ntot = Nsources
    print "Number of Sources Simulated = %f" % Ntot
    print "***********************"

        
#    ## -== Give names ==-
#    all_names = np.array(['']*Nsources,dtype='|S256')
#    #all_flux = np.zeros(Nsources)
#    for i in range(Nsources):  
#        src_name = '%sobs_%i' % (namepref,i)
#        all_names[i] = src_name
#        #all_flux[i] = flux

        
    ## -== Make the simput catalogue + spectrum ==-
    if(outfile != ''):
        print "... Writing SIMPUT catalogue ..."
        splength = np.size(energ)
        mysim = simput.simputobj(['catalogue','spectrum'],cat_columns=['SRC_NAME'],spec_columns=['NAME'],speclength=splength)
       
        # ~ Simput Spectrum ~
        print "  a) Registering spectra"
        specid = []
        spname = []
        i=1
        spid = '%sspecsrc%i' % (paramDICT['namepref'],i)
        spname.append(spid)
        for i in range(Nsources):
            specid.append("[SPECTRUM,1][NAME=='%s']" % spid)
        mysim.spectrum.data["NAME"] = np.array(spname)
        mysim.spectrum.data["ENERGY"] = energ
        mysim.spectrum.data["FLUX"] = spec
        mysim.spectrum.nspectra = 1
        



        # ~ Simput Catalogue ~
        print "  b) Registering sources"
        mysim.catalogue.data["SRC_ID"] = np.arange(Nsources)
        mysim.catalogue.data["SRC_NAME"] = all_names
        mysim.catalogue.data["RA"] = all_ra
        mysim.catalogue.data["DEC"] = all_dec
        mysim.catalogue.data["E_MIN"] = np.zeros(Nsources)+band[0]
        mysim.catalogue.data["E_MAX"] = np.zeros(Nsources)+band[1]
        mysim.catalogue.data["FLUX"] = all_flux
        mysim.catalogue.data["SPECTRUM"] = np.array(specid)
        mysim.catalogue.data["IMAGE"]  = np.array(['NULL']*Nsources)
        mysim.catalogue.data["TIMING"] = np.array(['NULL']*Nsources)

        print outfile
        print "  c) Writing file"
        mysim.writeto(outfile)



def age_sixterun(simput, prefix, background, input_attitude, output_attitude, area_def, area_type, exposure):

    """

    by AGE

    run sixte from python, sixte v.2.2.0, simput v.2.2.0

    """

    # -== Positions and area ==-
    if(area_type == 'angularbox'):
        ramin = area_def[0][0]*pi/180.
        ramax = area_def[0][1]*pi/180.
        decmin = area_def[1][0]*pi/180.
        decmax = area_def[1][1]*pi/180.
        print "Do not know how to run sixte with angularbox"
        sys.exit()
    elif(area_type == 'projcircle'):
        ra_cen = area_def[0]
        dec_cen = area_def[1]
        radius = area_def[2]
    else:
        print "Error: must specify area_type"
        sys.exit(1)

    time0=0
    if( input_attitude!='' and  output_attitude!=''):
        time0, exposure = mk_attitude(master_attitude=input_attitude, area_def=area_def, geomtype=area_type, output_attitude=output_attitude)
        #print time0, exposure
        #sys.exit()
    if(input_attitude!='' and  output_attitude=='' and exposure==0):
        output_attitude = input_attitude
        h= pf.open(output_attitude )
        time0 = h[1].data['TIME'][0]
        exposure = h[1].data['TIME'][-1] - h[1].data['TIME'][0]
        h.close()
    if(input_attitude!='' and  output_attitude=='' and exposure>0):
        time=np.arange(0,exposure+10,10)
        ra=ra_cen + 0.0* time
        dec=dec_cen + 0.0* time
        roll1=0.0 + 0.0* time        
        col0=pf.Column(name='TIME', format='D', array=time)
        col1=pf.Column(name='RA', format='D', array=ra)
        col2=pf.Column(name='DEC', format='D', array=dec)
        col3=pf.Column(name='ROLL1', format='D', array=roll1)

        cols = pf.ColDefs([col0, col1, col2, col3])
        dict = {"EXTNAME" : "ATT", "TSTART" : time[0], "TSTOP" : time[-1], "MJDREF": 51543.875, "UTCZERO" : 946674000}
        header=init_src_header(dict)

 
        
        hdu=pf.BinTableHDU.from_columns(cols, header=header, name="ATT")
        if(output_attitude==''):
            output_attitude="fixed_attitude.fits"
        hdu.writeto(output_attitude, clobber=True)



    erosimLIST=[]
    ero_caleventsLIST=[]

    task = 'erosim Prefix=\"' + prefix +'\"' 
    task += ' RA=%f' % (ra_cen) 
    task += ' Dec=%f' % (dec_cen) 
    for i in range(1,8):
        
        #task = 'erosim Prefix=\"' + prefix +'\"' 
        #task += ' RA=%f' % (ra_cen) 
        #task += ' Dec=%f' % (dec_cen) 
        task += ' XMLFile'+str(i)+'=\"/soft9/simput/share/sixte/instruments/srg/erosita_'+str(i)+'.xml\"' 
        #task += ' Simput=\"' + simput + '\"'
        #task += ' Simput2=\"' + background +'\"'
        #task += ' Attitude=\"' + output_attitude + '\"'
        #task += ' TSTART=%i' % (time0)
        #task += ' Exposure=%i' % (exposure)
        #task += ' clobber=yes' 

        #erosimLIST.append(task)

        task1 = 'ero_calevents EvtFile=\"%sccd%s_evt.fits\"' % (prefix,i) 
        task1 += ' eroEvtFile=\"%sccd%s_new_events.fits\"' % (prefix,i) 
        task1 += ' CCDNr=%s' % i 
        task1 += ' RefRA=%f' % (ra_cen) 
        task1 += ' RefDec=%f' % (dec_cen) 
        task1 += ' RA=%f' % (ra_cen) 
        task1 += ' Dec=%f' % (dec_cen) 
        task1 += ' Projection=TAN'
        task1 += ' Attitude=\"' + output_attitude + '\"'
        task1 += ' clobber=yes' 

        ero_caleventsLIST.append(task1)
        
    task += ' Simput=\"' + simput + '\"'
    task += ' Simput2=\"' + background +'\"'
    task += ' Attitude=\"' + output_attitude + '\"'
    task += ' TSTART=%i' % (time0)
    task += ' Exposure=%i' % (exposure)
    task += ' clobber=yes' 
    erosimLIST.append(task)

    return erosimLIST,ero_caleventsLIST
        



def call(*args):
    import subprocess

    #arguments = locals()
    command=""
    for i in range(len(args)):
        command = command + args[i] + " "

    print "Executing:",command
    proc = subprocess.Popen(command, shell=True, stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    proc.wait()

            
def sample_attitude(master_attitude="", output_attitude="", sample_step=10):

    hdu = pf.open(master_attitude)
    header1 = hdu[1].header
    ra = hdu[1].data['RA'][0::sample_step]
    dec = hdu[1].data['DEC'][0::sample_step]
    time = hdu[1].data['TIME'][0::sample_step]
    roll = hdu[1].data['ROLL'][0::sample_step]

    col1 = pf.Column(name="TIME",format='D',array=time)
    col2 = pf.Column(name="RA",format='D',array=ra)
    col3 = pf.Column(name="DEC",format='D',array=dec)
    col4 = pf.Column(name="ROLL",format='D',array=roll)

    #dict = {"EXTNAME" : "ATT", "TSTART" : time[0], "TSTOP" : time[-1], "MJDREF": 51543.875, "UTCZERO" : 946674000}
    header1["TSTART"]=time[0]
    header1["TSTOP"]=time[-1]

    colarray = [col1,col2,col3,col4]
    pfcols = pf.ColDefs(colarray)
    out = pf.BinTableHDU.from_columns(pfcols, header=header1, name="ATT")
    out.writeto(output_attitude, clobber=True)


    
def mk_attitude(master_attitude, area_def, geomtype, output_attitude):
    """

    by AGE

    generate a attitude file from an input one by selecting only 
    the ra, dec of interest with some radius off the center. 


    UPDATES:

    """

    print master_attitude, output_attitude
    
    hdu = pf.open(master_attitude)
    ra = hdu[1].data['RA']
    dec = hdu[1].data['DEC']
    time = hdu[1].data['TIME']
    roll = hdu[1].data['ROLL']

    dt = (time[1:time.size] - time[0:time.size-1])[0]
    
    
    
    if(geomtype == 'angularbox'):

        decmin = area_def[1][0] - 2.0
        decmax = area_def[1][1] + 2.0
        dec_cen = 0.5 * (decmin+ decmax)
        ramin = area_def[0][0] - 2.0 / numpy.cos(dec_cen * 180./pi)
        ramax = area_def[0][1] + 2.0 / numpy.cos(dec_cen * 180./pi)

        mask1 = ra > ramin
        mask2 = ra < ramax
        mask3 = dec > decmin
        mask4 = dec < decmax

        mask = numpy.logical_and(mask1, maks2)
        mask = numpy.logical_and(mask, maks3)
        mask = numpy.logical_and(mask, maks4)
        
        
    if(geomtype == 'projcircle'):
        ra_cen = area_def[0]
        dec_cen = area_def[1]
        radius = area_def[2] * 0.5

        #print ra_cen, dec_cen, radius

        mydist = mydist_on_sphere(ra,dec,ra_cen,dec_cen)

        mask = mydist < radius

    ra = ra[mask]
    dec = dec[mask]
    time = time[mask]
    roll = roll[mask]

    
    time = range(time.size)
    
    time = (np.array(time)).astype(float) * dt 

    dict = {"EXTNAME" : "ATT", "TSTART" : time[0], "TSTOP" : time[-1], "MJDREF": 51543.875, "UTCZERO" : 946674000}
    header=init_src_header(dict)


    
    col1 = pf.Column(name="TIME",format='D',array=time)
    col2 = pf.Column(name="RA",format='D',array=ra)
    col3 = pf.Column(name="DEC",format='D',array=dec)
    col4 = pf.Column(name="ROLL1",format='D',array=roll)

    colarray = [col1,col2,col3,col4]
    pfcols = pf.ColDefs(colarray)
    out = pf.BinTableHDU.from_columns(pfcols, header=header, name="ATT")
    out.writeto(output_attitude, clobber=True)

    return time[0], time[-1];


def init_src_header(dict):
    
    header = pf.Header()
    for row in dict.keys():
        header[row] = (dict[row])
    return header;

