#! /usr/bin/env python

###########################################################
#
#  Creates spectral templates according to Gilli 2007
#
#  See also:
#  https://wiki.mpe.mpg.de/eRosita/SyntheticSimulations?action=AttachFile&do=view&target=gilli_implementation_20130814.pdf
#
#  Author: N.Clerc / MPE
#  Contributors: C.Schmid, T.Brand / Sternwarte Bamberg
#
#  Created: 04/05/2012
#
#  History: 06/01/2017 -- Compatibility with Pyfits-v3.3 methods
#
#
###########################################################

"""
Contains functions for creating Gilli templates from xspec
Only produces z=0 templates free of galactic absorption.
"""

import os,sys,time
import pyfits as pf
import numpy as np
import string
import subprocess
from scipy import interpolate
from math import log10
import spectrotools as sptools

abund = "angr"

#----------

def launchxspec(filename,dirtemp,spectempname,totofak,logfile):

    """
    Launch XSPEC (pay attention to your version of Xspec)
    Relies on the Popen command via shell -> compatibility with all OS is not ensured.
    """

    ## Get XSPEC path
    xspecbin = "${HEADAS}/bin/xspec"

    ## Remove any existing spectra of the same name
    if(os.path.exists(dirtemp+'/'+spectempname)):
        os.remove(dirtemp+'/'+spectempname)

    ## Launch XSPEC
    command = xspecbin+" < "+dirtemp+"/"+filename
    run = subprocess.Popen(command,shell=True,stderr=subprocess.STDOUT,stdout=subprocess.PIPE)
    commu = run.communicate()
    commustr = commu[0]

    # Log Xspec
    if(os.path.exists(dirtemp+'/'+logfile)):
        os.remove(dirtemp+'/'+logfile)
    outFile=file(dirtemp+"/"+logfile,'w')
    outFile.writelines(commustr)
    outFile.close()
    
    ## Remove junk
    os.remove(dirtemp+"/"+filename)
    if(os.path.exists(dirtemp+"/"+totofak)):
           os.remove(dirtemp+"/"+totofak)

#----------

def get_xsspectrum(dirtemp,specname,verbose=0):

    """
    Reads a spectrum file issued by XSpec
    """
    
    spec = file("%s/%s" % (dirtemp,specname)).readlines()
    energy = []
    model = []

    for i in range(len(spec)):
        if(i < 3):
            continue
        line = spec[i]
        items = line.split()
        items = map(string.strip,items)
        energy.append(float(items[0]))
        model.append(float(items[2]))

    if(verbose > 2):print " (%i lines read)" % i
    return (np.array(energy),np.array(model))
#----------
def gilli_eqwidth(nh):

    """
    Implements the equivalent width of Fe as a function of N_H according to Gilli+07
    """

    # Form Tab 4 in Gilli 99
    lognhval = [21,21.5,22.5,23.5,24]
    wk = [0.280,0.370,0.300,0.470,2.100]
    
    lognh = log10(nh)
    if(lognh < 21):
        return 0.280
    elif(lognh > 24):
        return 2.100
    else:
        f=interpolate.interp1d(lognhval,wk)
        return f(lognh)



#----------
def mk_pl(dirtemp,phoindex,npoints=2000,verbose=0,addstring=''):

    """
    Creates and a porerlaw spectrum with phoindex
     * dirtemp: temporary directory for I/O
     * phoindex: photon index as defined in Xspec Power-Law template
     * npoints: number of energy data points
     * addstring: a generic string to append all temporary files

    modified by AGE based on Clercs routines
    """
    
    randid = np.random.randint(1000000,9999999)
    filename = addstring+"xspec_script_gen_%i.xcm" % randid
    totofak  = addstring+"toto_%i.fak" % randid
    specres  = addstring+"specfile_%i.fak" % randid
    logfile  = addstring+"logfile_%i.log" % randid

    print dirtemp
    print addstring
    print filename

    
    lines=[]
    lines.append("abund %s" % abund)
    lines.append("setplot device /null")
    lines.append("setplot energy")
    lines.append("setplot noadd")
    lines.append("energies 0.1 1000 %i log" % npoints)

    model = "model power "
    model += "& %f  " % (phoindex) # Slope of powerlaw
    model += "& 1.0 "    # normalisation
    lines.append(model)
    lines.append("fakeit 1 & & y & & %s/%s & &" % (dirtemp,totofak) )
    
    
    lines.append("show par") # Just to check in the LOG
    lines.append("plot mod")
    lines.append("iplot")
    lines.append('wdata "%s/%s"' % (dirtemp,specres))
    lines.append("exit")


    
    outFile = file(dirtemp+"/"+filename,'w')
    text = map(lambda x: '%s\n' %x,lines)
    outFile.writelines(text)
    outFile.close()

    print dirtemp+"/"+filename
    
    
    launchxspec(filename,dirtemp,specres,totofak,logfile)
    return get_xsspectrum(dirtemp,specres,verbose=verbose)
    


    
#----------
def mk_unobscured(dirtemp,phoindex,qsoregime=False,npoints=2000,verbose=0,addstring=''):

    """
    Creates and unobscured AGN template
     * dirtemp: temporary directory for I/O
     * phoindex: photon index as defined in Xspec Power-Law template
     * qsoregime: True/False, indicates whether the object is in QSO regime (see Gilli+07)
     * npoints: number of energy data points
     * addstring: a generic string to append all temporary files
    """

    ## logNH<21

    randid = np.random.randint(1000000,9999999)
    filename = addstring+"xspec_script_gen_%i.xcm" % randid
    totofak  = addstring+"toto_%i.fak" % randid
    specres  = addstring+"specfile_%i.fak" % randid
    logfile  = addstring+"logfile_%i.log" % randid

    lines=[]
    lines.append("abund %s" % abund)
    lines.append("setplot device /null")
    lines.append("setplot energy")
    lines.append("setplot noadd")
    lines.append("energies 0.1 1000 %i log" % npoints)

    if(not qsoregime):
        model = "model reflect*(cutoffpl+gauss) "
        model += "& 1.3 "    # norm of reflected component
        model += "& 0.0 "    # redshift
        model += "& 1.0 "    # abund
        model += "& 1.0 "    # Fe abund
        model += "& 0.45 "   # cosi
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 200 "    # 200keV cutoff
        model += "& 1.0 "    # norm
        model += "& 6.4 "    # position of Fe
        model += "& 0.4 "    # width of Fe
        model += "& 1.0 "      # temporary norm of gaussian
        lines.append(model)
        lines.append("fakeit 1 & & y & & %s/%s & &" % (dirtemp,totofak) )
        lines.append("eqwidth 3")
        lines.append("tclout eqwidth 1")
        lines.append("set width [lindex $xspec_tclout 0]")
        lines.append("newpar 11 [expr 0.280/${width}]")
        lines.append("eqwidth 3") # Just to check in the LOG

    
    if(qsoregime):
        model = "model cutoffpl "
        model += "& %f " % (phoindex) # Slope of powelaw
        model += "& 200 " # 200 keV cutoff
        model += "& 1.0 "   # norm
        lines.append(model)
    
    lines.append("show par") # Just to check in the LOG
    lines.append("plot mod")
    lines.append("iplot")
    lines.append('wdata "%s/%s"' % (dirtemp,specres))
    lines.append("exit")
    
    outFile = file(dirtemp+"/"+filename,'w')
    text = map(lambda x: '%s\n' %x,lines)
    outFile.writelines(text)
    outFile.close()

    launchxspec(filename,dirtemp,specres,totofak,logfile)
    return get_xsspectrum(dirtemp,specres,verbose=verbose)
    


#----------
def mk_comptonthin(dirtemp,phoindex,nh,qsoregime=False,npoints=2000,verbose=0,addstring=''):

    """ 
    Creates a Compton-Thin AGN template
     * dirtemp: temporary directory for I/O
     * phoindex: photon index as defined in Xspec Power-Law template
     * qsoregime: True/False, indicates whether the object is in QSO regime (see Gilli+07)
     * npoints: number of energy data points
     * addstring: a generic string to append all temporary files     
    """

    ## 21<logNH<24
    if(nh > 1e24 or nh <= 1e21):
        print "NH value not good for Compton Thin models"
        print "aborting"
        sys.exit(1)

    randid = np.random.randint(1000000,9999999)
    filename = addstring+"xspec_script_gen_%i.xcm" % randid
    totofak  = addstring+"toto_%i.fak" % randid
    specres  = addstring+"specfile_%i.fak" % randid
    logfile  = addstring+"logfile_%i.log" % randid

    lines=[]
    lines.append("abund %s" % abund)
    lines.append("setplot device /null")
    lines.append("setplot energy")
    lines.append("setplot noadd")
    lines.append("energies 0.1 1000 %i log" % npoints)

    if(not qsoregime):
        model = "model wabs*(reflect*(cutoffpl+gauss))+cutoffpl "
        model += "& %f " % (nh/1e22)    # Nh in 10^22
        model += "& 0.88 "   # norm of reflected component
        model += "& 0.0 "    # redshift
        model += "& 1.0 "    # abund
        model += "& 1.0 "    # Fe abund
        model += "& 0.45 "   # cosi
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 200 "    # 200keV cutoff
        model += "& 1.0 "    # norm
        model += "& 6.4 "    # position of Fe
        model += "& 0.1 "    # width of Fe
        model += "& 1.0 "    # temporary norm of gaussian
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 3."    # Not clear
        model += "& 0.03 "   # 3% according to Gilli
        lines.append(model)
        lines.append("fakeit 1 & & y & & %s/%s & &" % (dirtemp,totofak) )
        lines.append("eqwidth 4")
        lines.append("tclout eqwidth 1")
        lines.append("set width [lindex $xspec_tclout 0]")
        myeqwidth = gilli_eqwidth(nh)    
        lines.append("newpar 12 [expr %f/${width}]" % myeqwidth)
        lines.append("eqwidth 4") # Just to check in the LOG

    
    if(qsoregime):
        model = "model wabs*(cutoffpl)+cutoffpl "
        model += "& %f " % (nh/1e22)    # Nh in 10^22
        model += "& %f " % (phoindex) # Slope of powelaw
        model += "& 200 " # 200 keV cutoff
        model += "& 1.0 " # norm
        model += "& %f " % (phoindex) # Slope of powelaw
        model += "& 3." # Not clear
        model += "& 0.03" # 3% according to Gilli
        lines.append(model)
    
    lines.append("show par") # Just to check in the LOG
    lines.append("plot mod")
    lines.append("iplot")
    lines.append('wdata "%s/%s"' % (dirtemp,specres))
    lines.append("exit")
    
    outFile = file(dirtemp+"/"+filename,'w')
    text = map(lambda x: '%s\n' %x,lines)
    outFile.writelines(text)
    outFile.close()

    launchxspec(filename,dirtemp,specres,totofak,logfile)
    return get_xsspectrum(dirtemp,specres,verbose=verbose)


#----------
def mk_comptonthickmild(dirtemp,phoindex,nh,qsoregime=False,npoints=2000,verbose=0,addstring=''):

    """ 
    Creates a Compton-Thick-Midl AGN template
     * dirtemp: temporary directory for I/O
     * phoindex: photon index as defined in Xspec Power-Law template
     * qsoregime: True/False, indicates whether the object is in QSO regime (see Gilli+07)
     * npoints: number of energy data points
     * addstring: a generic string to append all temporary files     
    """

    ## 24<logNH<25
    if(nh > 1e25 or nh <= 1e24):
        print "NH value not good for Compton Thick 'mild' models"
        print "aborting"
        sys.exit(1)

    randid = np.random.randint(1000000,9999999)
    filename = addstring+"xspec_script_gen_%i.xcm" % randid
    totofak  = addstring+"toto_%i.fak" % randid
    specres  = addstring+"specfile_%i.fak" % randid
    logfile  = addstring+"logfile_%i.log" % randid

    lines=[]
    lines.append("abund %s" % abund)
    lines.append("setplot device /null")
    lines.append("setplot energy")
    lines.append("setplot noadd")
    lines.append("energies 0.1 1000 %i log" % npoints)

    if(not qsoregime):
        model = "model reflect*(plcabs+gauss)+reflect*(cutoffpl)+cutoffpl"
        # mod
        model += "& 0.88 "   # norm of reflected component (disk)
        model += "& 0.0 "    # redshift
        model += "& 1.0 "    # abund
        model += "& 1.0 "    # Fe abund
        model += "& 0.45 "   # cosi
        # mod
        model += "& %f " % (nh/1e22)    # Nh in 10^22
        model += "& 5 "      # nmax
        model += "& "        # FeAbund
        model += "& "        # FeKedge
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& .01 "    # Ecut
        model += "& 400."    # Initial PL
        model += "& 0.1"     # acrit
        model += "& "        # FAST
        model += "& 0.0 "    # Redshift
        model += "& 1.0 "    # Norm
        # mod
        model += "& 6.4 "    # position of Fe
        model += "& 0.1 "    # width of Fe
        model += "& 1.0 "    # temporary norm of gaussian
        # mod
        model += "& -0.37 "  # norm of reflected component (torus)
        model += "& 0.0 "    # redshift
        model += "& 1.0 "    # abund
        model += "& 1.0 "    # Fe abund
        model += "& 0.45 "   # cosi
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 200."    # Initial PL
        model += "& 1.0 "    # Initial PL
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 2.7"    # Not clear
        model += "& 0.03 "   # 3% according to Gilli
        lines.append(model)
        lines.append("fakeit 1 & & y & & %s/%s & &" % (dirtemp,totofak) )
        lines.append("eqwidth 3")
        lines.append("tclout eqwidth 1")
        lines.append("set width [lindex $xspec_tclout 0]")
        myeqwidth = gilli_eqwidth(nh)    
        lines.append("newpar 19 [expr %f/${width}]" % myeqwidth)
        lines.append("eqwidth 3") # Just to check in the LOG

    
    if(qsoregime):
        model = "model plcabs+reflect*(cutoffpl)+cutoffpl"

        # mod
        model += "& %f " % (nh/1e22)    # Nh in 10^22
        model += "& 5 "      # nmax
        model += "& "        # FeAbund
        model += "& "        # FeKedge
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& .01 "    # Ecut
        model += "& 400."    # Initial PL
        model += "& 0.1"     # acrit
        model += "& "        # FAST
        model += "& 0.0 "    # Redshift
        model += "& 1.0 "    # Norm
        # mod
        model += "& -0.37 "  # norm of reflected component (torus)
        model += "& 0.0 "    # redshift
        model += "& 1.0 "    # abund
        model += "& 1.0 "    # Fe abund
        model += "& 0.45 "   # cosi
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 200."    # Initial PL
        model += "& 1.0 "    # Initial PL
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 2.7"    # Not clear
        model += "& 0.03 "   # 3% according to Gilli
        lines.append(model)
    
    lines.append("show par") # Just to check in the LOG
    lines.append("plot mod")
    lines.append("iplot")
    lines.append('wdata "%s/%s"' % (dirtemp,specres))
    lines.append("exit")
    
    outFile = file(dirtemp+"/"+filename,'w')
    text = map(lambda x: '%s\n' %x,lines)
    outFile.writelines(text)
    outFile.close()

    launchxspec(filename,dirtemp,specres,totofak,logfile)
    return get_xsspectrum(dirtemp,specres,verbose=verbose)

#----------
def mk_comptonthickheavy(dirtemp,phoindex,nh,qsoregime=False,npoints=2000,verbose=0,addstring=''):

    """ 
    Creates a Compton-Thick-Heavy AGN template
     * dirtemp: temporary directory for I/O
     * phoindex: photon index as defined in Xspec Power-Law template
     * qsoregime: True/False, indicates whether the object is in QSO regime (see Gilli+07)
     * npoints: number of energy data points
     * addstring: a generic string to append all temporary files     
    """

    ## logNH>25
    if(nh <= 1e25):
        print "NH value not good for Compton Thick 'heavy' models"
        print "aborting"
        sys.exit(1)

    randid = np.random.randint(1000000,9999999)
    filename = addstring+"xspec_script_gen_%i.xcm" % randid
    totofak  = addstring+"toto_%i.fak" % randid
    specres  = addstring+"specfile_%i.fak" % randid
    logfile  = addstring+"logfile_%i.log" % randid

    lines=[]
    lines.append("abund %s" % abund)
    lines.append("setplot device /null")
    lines.append("setplot energy")
    lines.append("setplot noadd")
    lines.append("energies 0.1 1000 %i log" % npoints)

    if(not qsoregime):
        model = "model reflect*(cutoffpl+gauss)+cutoffpl"
        # mod
        model += "& -0.37 "  # norm of reflected component (torus)
        model += "& 0.0 "    # redshift
        model += "& 1.0 "    # abund
        model += "& 1.0 "    # Fe abund
        model += "& 0.45 "   # cosi
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 300."    # Initial PL
        model += "& 1.0 "    # Initial PL
        # mod
        model += "& 6.4 "    # position of Fe
        model += "& 0.1 "    # width of Fe
        model += "& 1.0 "    # temporary norm of gaussian
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 2.7"    # Not clear
        model += "& 0.03 "   # 3% according to Gilli
        lines.append(model)
        lines.append("fakeit 1 & & y & & %s/%s & &" % (dirtemp,totofak) )
        lines.append("eqwidth 3")
        lines.append("tclout eqwidth 1")
        lines.append("set width [lindex $xspec_tclout 0]")
        myeqwidth = gilli_eqwidth(nh)    
        lines.append("newpar 11 [expr %f/${width}]" % myeqwidth)
        lines.append("eqwidth 3") # Just to check in the LOG

    
    if(qsoregime):
        model = "model reflect*(cutoffpl)+cutoffpl"
        # mod
        model += "& -0.37 "  # norm of reflected component (torus)
        model += "& 0.0 "    # redshift
        model += "& 1.0 "    # abund
        model += "& 1.0 "    # Fe abund
        model += "& 0.45 "   # cosi
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 200."    # Initial PL
        model += "& 1.0 "    # Initial PL
        # mod
        model += "& %f  " % (phoindex) # Slope of powerlaw
        model += "& 3."    # Not clear
        model += "& 0.03 "   # 3% according to Gilli    
        lines.append(model)
    
    lines.append("show par") # Just to check in the LOG
    lines.append("plot mod")
    lines.append("iplot")
    lines.append('wdata "%s/%s"' % (dirtemp,specres))
    lines.append("exit")
    
    outFile = file(dirtemp+"/"+filename,'w')
    text = map(lambda x: '%s\n' %x,lines)
    outFile.writelines(text)
    outFile.close()

    launchxspec(filename,dirtemp,specres,totofak,logfile)
    return get_xsspectrum(dirtemp,specres,verbose=verbose)

#----------
def mk_gillimodel(dirtemp,phoindex,nh,z,luminosity,bandlum=[0.5,2],npoints=2000,verbose=0,addstring='',QSO_thresh=10**(43.5)):
    """
    Wrapper deciding which of the above functions is to be used according to
     * photon index
     * intrinsic nH (units cm**-2)
     * (rest-frame) luminosity (units ergs/s) in band bandlum (units keV)
     * redshift

     Returns the corresponding spectrum. No normalization of any kind.
     Units photons/s/kev/cm**2
     Beware of the output energy range : [0.1 - 1000.]/(1+z) (keV)
    """

    #QSO_thresh = 10**(43.5) ## Seyfert/QSO regime separation, 0.5-2 keV

    # 0. Compute the Unabsorbed spectrum, necessary for evaluating intrinsic luminosity
    energqso_deabs,qsomodel_deabs=mk_unobscured(dirtemp,phoindex,qsoregime=True,npoints=npoints,verbose=verbose,addstring='deabs_'+addstring)
    energsey_deabs,seymodel_deabs=mk_unobscured(dirtemp,phoindex,qsoregime=False,npoints=npoints,verbose=verbose,addstring='deabs_'+addstring)
    
    # 1. Select NH
    logNH = log10(nh)

    if(logNH <= 21):
        if(verbose > 0):print "AGN model (%s): unobscured" % addstring
        energqso = energqso_deabs
        qsomodel = qsomodel_deabs
        energsey = energsey_deabs
        seymodel = seymodel_deabs
    elif(logNH <= 24):
        if(verbose > 0):print "AGN model (%s): Compton Thin" % addstring
        energqso,qsomodel = mk_comptonthin(dirtemp,phoindex,nh,qsoregime=True,npoints=npoints,verbose=verbose,addstring=addstring)
        energsey,seymodel = mk_comptonthin(dirtemp,phoindex,nh,qsoregime=False,npoints=npoints,verbose=verbose,addstring=addstring)
    elif(logNH <= 25):
        if(verbose > 0):print "AGN model (%s): Compton Thick/mild" % addstring
        energqso,qsomodel = mk_comptonthickmild(dirtemp,phoindex,nh,qsoregime=True,npoints=npoints,verbose=verbose,addstring=addstring)
        energsey,seymodel = mk_comptonthickmild(dirtemp,phoindex,nh,qsoregime=False,npoints=npoints,verbose=verbose,addstring=addstring)
    else:
        if(verbose > 0):print "AGN model (%s): Compton Thick/heavy" % addstring
        energqso,qsomodel = mk_comptonthickheavy(dirtemp,phoindex,nh,qsoregime=True,npoints=npoints,verbose=verbose,addstring=addstring)
        energsey,seymodel = mk_comptonthickheavy(dirtemp,phoindex,nh,qsoregime=False,npoints=npoints,verbose=verbose,addstring=addstring)

    # 2. Select QSO/Seyfert (Compute on DE-absorbed spectra)
    norm_052_qso = sptools.integ_spectr(energqso_deabs,qsomodel_deabs,0.5,2.0)
    norm_052_sey = sptools.integ_spectr(energsey_deabs,seymodel_deabs,0.5,2.0)
    
    norm_bnd_qso = sptools.integ_spectr(energqso_deabs,qsomodel_deabs,bandlum[0],bandlum[1])
    norm_bnd_sey = sptools.integ_spectr(energsey_deabs,seymodel_deabs,bandlum[0],bandlum[1])

    lum_052_qso = luminosity*(norm_052_qso/norm_bnd_qso)
    lum_052_sey = luminosity*(norm_052_sey/norm_bnd_sey)

    good = sptools.check_seyqso(lum_052_sey,lum_052_qso,QSO_thresh=QSO_thresh)


    if(verbose > 0):print "AGN model (%s): %s" % (addstring,good)
    if(good == "sey"):
        energ,spectrum = energsey.copy(),seymodel.copy()
    else:
        energ,spectrum = energqso.copy(),qsomodel.copy()

    if(verbose>2): print ""
    # 3. Redshift spectrum
    return sptools.redsh_spectr(energ,spectrum,z)

#-----------
def read_specbank(bankname,phoindex,nh,z,luminosity,bandlum=[0.5,2.0],QSO_thresh=10**(43.5)):
    """
    Read a spectral bank created with create_specbank
    
    * bankname  : bank to be read
    * phoindex  : array of photon indexes
    * nh        : array of nh values
    * z         : array of redshifts
    * luminosity: array of luminosities (for Sey/QSO separation)
    * bandlum   : corresponding luminosity
    * QSO_thresh: luminosity for choosing a QSO rather that a Sey model
    
    Result is 6 lists: one for the energy arrays, the other for
    the spectral arrays, one list consisting of the type (sey,qso).
    The latter three correspond to the 0.5-2 ; 2-10 ; 10-1000 normalizations
    of the corresponding De-absorbed spectrum.


    Warning: No normalization of spectra according to luminosity
    (must be done afterwards through the provided normalization factors)
    """
    
    print " > opening spectral Bank: %s" % bankname
    ## Open bank file
    datasey = pf.open(bankname)[1].data
    dataqso = pf.open(bankname)[2].data

    ## Check array sizes
    npho = np.size(phoindex)
    nnh  = np.size(nh)
    nz   = np.size(z)
    nlum = np.size(luminosity)
    
    nspec = max([npho,nnh,nz,nlum])
    if(npho == 1):
        phovec = np.zeros(nspec)+phoindex
    elif(npho != nspec):
        print "Size mismatch (phoindex)"
        sys.exit(1)
    else:
        phovec = phoindex.copy()
        
    if(nnh == 1):
        nhvec = np.zeros(nspec)+nh
    elif(nnh != nspec):
        print "Size mismatch (nh)"
        sys.exit(1)
    else:
        nhvec = nh.copy()

    if(nz == 1):
        zvec = np.zeros(nspec)+z
    elif(nz != nspec):
        print "Size mismatch (z)"
        sys.exit(1)
    else:
        zvec = z.copy()

    if(nlum == 1):
        lumvec = np.zeros(nspec)+luminosity
    elif(nlum != nspec):
        print "Size mismatch (luminosity)"
        sys.exit(1)
    else:
        lumvec = luminosity.copy()


    ## Loop over each spectrum
    energies = []
    spectra  = []
    goodlist = []
    norm052list = []
    norm210list = []
    norm101000list = []


    for i in range(nspec):

	# Apply condition
        seycond = (phovec[i] >= datasey.field("gammamin")) * (phovec[i] < datasey.field("gammamax")) * (nhvec[i] >= datasey.field("nhmin")) * (nhvec[i] < datasey.field("nhmax"))
        qsocond =  (phovec[i] >= dataqso.field("gammamin")) * (phovec[i] < dataqso.field("gammamax")) * (nhvec[i] >= dataqso.field("nhmin")) * (nhvec[i] < dataqso.field("nhmax"))

        temp, = np.where(seycond)
        seypos = temp[0]
        temp, = np.where(qsocond)
        qsopos = temp[0]

       	# Check unicity
        if(np.size(seypos) != 1):
            print "Seyfert bank problem"
            print " Gamma=%f" % phovec[i]
            print " Nh=%f" % nhvec[i]
            sys.exit(1)
        if(np.size(qsopos) != 1):
            print "QSO bank problem"
            print " Gamma=%f" % phovec[i]
            print " Nh=%f" % nhvec[i]
            sys.exit(1)

	
        #print "Ask: %f ; got %f - (%f) - %f" %(phovec[i], datasey.field("gammamin")[seypos], datasey.field("gammamed")[seypos],datasey.field("gammamax")[seypos])
        #print "Ask: %f ; got %f - (%f) - %f" %(nhvec[i], datasey.field("nhmin")[seypos], datasey.field("nhmed")[seypos],datasey.field("nhmax")[seypos])

	
        energsey  = datasey.field("energy")[seypos][:]
        seymodel = datasey.field("spectrum")[seypos][:]
        energqso = dataqso.field("energy")[qsopos][:]
        qsomodel = dataqso.field("spectrum")[qsopos][:]
        energsey_deabs = datasey.field("deabs_energy")[seypos][:]
        seymodel_deabs = datasey.field("deabs_spectrum")[seypos][:]
        energqso_deabs = dataqso.field("deabs_energy")[qsopos][:]
        qsomodel_deabs = dataqso.field("deabs_spectrum")[qsopos][:]

        # Select QSO/Seyfert
        norm_052_qso = dataqso.field("norm052")[qsopos]
        norm_052_sey = datasey.field("norm052")[seypos]
       
        norm_bnd_qso = sptools.integ_spectr(energqso_deabs,qsomodel_deabs,bandlum[0],bandlum[1])
        norm_bnd_sey = sptools.integ_spectr(energsey_deabs,seymodel_deabs,bandlum[0],bandlum[1])
        
        lum_052_qso = lumvec[i]*(norm_052_qso/norm_bnd_qso)
        lum_052_sey = lumvec[i]*(norm_052_sey/norm_bnd_sey)

        good = sptools.check_seyqso(lum_052_sey,lum_052_qso,QSO_thresh=QSO_thresh)
                
        ##
        #print seypos,qsopos
        #print norm_052_sey, norm_052_sey
        #print luminosity[i], lum_052_sey, lum_052_qso
        #print good
        ##       

        if(good == "sey"):
            energ,spectrum = energsey.copy(),seymodel.copy()
            deabs_energ,deabs_spectrum = energsey_deabs.copy(),seymodel_deabs.copy()
            #n052 = datasey.field("norm052")[seypos]
            #n210 = datasey.field("norm210")[seypos]
            #n101000 = datasey.field("norm101000")[seypos]
        else:
            energ,spectrum = energqso.copy(),qsomodel.copy()
            deabs_energ,deabs_spectrum = energqso_deabs.copy(),qsomodel_deabs.copy()
            #n052 = dataqso.field("norm052")[qsopos]
            #n210 = dataqso.field("norm210")[qsopos]
            #n101000 = dataqso.field("norm101000")[qsopos]


        e,s =  sptools.redsh_spectr(energ,spectrum,zvec[i])
        deabs_e,deabs_s = sptools.redsh_spectr(deabs_energ,deabs_spectrum,zvec[i])
        # Compute rest-frames norms (!! redsh_spectr applies an arbitrary normalization !!)
        #   on intrinsic spectra
        n052 = sptools.integ_spectr(deabs_e,deabs_s,0.5/(1.+zvec[i]),2.0/(1.+zvec[i]))
        n210 = sptools.integ_spectr(deabs_e,deabs_s,2.0/(1.+zvec[i]),10.0/(1.+zvec[i]))
        n101000 = sptools.integ_spectr(deabs_e,deabs_s,10.0/(1.+zvec[i]),1000.0/(1.+zvec[i]))

        energies.append(e)
        spectra.append(s)
        goodlist.append(good)
        norm052list.append(n052)
        norm210list.append(n210)
        norm101000list.append(n101000)

    return energies,spectra,goodlist,norm052list,norm210list,norm101000list





#------------
def create_specbank(bankname,dirtemp,gammamin,gammamax,ngammas,nhmin,nhmax,nnh,npoints=1000):

    """
    Creates a spectral bank of Seyfert and QSOs for different values of gamma and Nh
    * bankname: name of FITS file to be created
    * dirtemp : directory where XSpec computations are performed
    * gammamin: minimum value of gamma
    * gammamax: maximum value of gamma
    * ngammas : number of gamma bins
    * nhmin,nhmax,nnh: see gammamin,gammamax,ngammas
    * npoints : number of points for each spectrum

    Result is a FITS file : EXT#1 for Seyfert models, EXT#2 for QSO models
    Columns corresponding to:
     - gammamin,gammamax,gammamed : bin of gamma
     - nhmin, nhmax, nhmed : bin of nh value
     - energy, spectrum : model spectrum
     - deabs_energy, deabs_spectrum : model for the de-absorbed spectrum (i.e. same powerlaw normalization)
     - norm052, norm210, norm101000 : quick saves of normalization for the de-absorbed spectra in 0.5-2 ; 2-10 ; 10-1000 keV

     Example:
         create_specbank('toto_bank.fits','./tmp_bank/',1.05,2.75,50,1e-20,1e20,1,npoints=1000)
         will create a spectral band "toto_bank.fits" with 50 Gamma's in
         [1.05,2.75] and 1 Nh = 1e-20 (actually will be put artificially to
         logNh=0, as a standard here for unobscured sources (logNh<21).)
    """

    ## Design values
    gtemp = np.linspace(gammamin,gammamax,ngammas+1)
    gmin = gtemp[0:-1]
    gmax = gtemp[1:]
    
    lognhtemp = np.linspace(log10(nhmin),log10(nhmax),nnh+1)
    lnhmin = lognhtemp[0:-1]
    lnhmax = lognhtemp[1:]
    
    Nspectra = ngammas*nnh

    gammamin = np.resize(gmin,Nspectra)
    gammamax = np.resize(gmax,Nspectra)
    gammamed = 0.5*(gammamin+gammamax)
        
    lognhmin = np.zeros(Nspectra)
    lognhmax = np.zeros(Nspectra)
    for i in range(ngammas):
        for j in range(nnh):
            lognhmin[j*ngammas+i] = lnhmin[j]
            lognhmax[j*ngammas+i] = lnhmax[j]
            lognhmed = 0.5*(lognhmin+lognhmax)


    norm052_sey = np.zeros(Nspectra)
    norm052_qso = np.zeros(Nspectra)
    norm210_sey = np.zeros(Nspectra)
    norm210_qso = np.zeros(Nspectra)
    norm101000_sey = np.zeros(Nspectra)
    norm101000_qso = np.zeros(Nspectra)


    all_e_sey = np.zeros((Nspectra,npoints))
    all_s_sey = np.zeros((Nspectra,npoints))
    all_e_qso = np.zeros((Nspectra,npoints))
    all_s_qso = np.zeros((Nspectra,npoints))
    all_e_sey_deabs = np.zeros((Nspectra,npoints))
    all_s_sey_deabs = np.zeros((Nspectra,npoints))
    all_e_qso_deabs = np.zeros((Nspectra,npoints))
    all_s_qso_deabs = np.zeros((Nspectra,npoints))

    ## Create spectra
    for i in range(Nspectra):
        logNH = lognhmed[i]
        phoindex = gammamed[i]
        nh = 10.**logNH
       
        print " Bank entry: logNH=%f , phoindex=%f" % (logNH,phoindex)
        energqso_deabs,qsomodel_deabs=mk_unobscured(dirtemp,phoindex,qsoregime=True,npoints=npoints,addstring='deabs-bank_')
        energsey_deabs,seymodel_deabs=mk_unobscured(dirtemp,phoindex,qsoregime=False,npoints=npoints,addstring='deabs-bank_')
        if(logNH <= 21.):
            energqso = energqso_deabs
            qsomodel = qsomodel_deabs
            energsey = energsey_deabs
            seymodel = seymodel_deabs
        elif(logNH <= 24.):
            energqso,qsomodel = mk_comptonthin(dirtemp,phoindex,nh,qsoregime=True,npoints=npoints,addstring='bank_')
            energsey,seymodel = mk_comptonthin(dirtemp,phoindex,nh,qsoregime=False,npoints=npoints,addstring='bank_')
        elif(logNH <= 25.):
            energqso,qsomodel = mk_comptonthickmild(dirtemp,phoindex,nh,qsoregime=True,npoints=npoints,addstring='bank_')
            energsey,seymodel = mk_comptonthickmild(dirtemp,phoindex,nh,qsoregime=False,npoints=npoints,addstring='bank_')
        else:
            energqso,qsomodel = mk_comptonthickheavy(dirtemp,phoindex,nh,qsoregime=True,npoints=npoints,addstring='bank_')
            energsey,seymodel = mk_comptonthickheavy(dirtemp,phoindex,nh,qsoregime=False,npoints=npoints,addstring='bank_')



        all_e_sey[i,:] = energsey
        all_e_qso[i,:] = energqso
        all_s_sey[i,:] = seymodel
        all_s_qso[i,:] = qsomodel
        all_e_sey_deabs[i,:] = energsey_deabs
        all_e_qso_deabs[i,:] = energqso_deabs
        all_s_sey_deabs[i,:] = seymodel_deabs
        all_s_qso_deabs[i,:] = qsomodel_deabs

        norm052_qso[i] = sptools.integ_spectr(energqso_deabs,qsomodel_deabs,0.5,2.0)
        norm052_sey[i] = sptools.integ_spectr(energsey_deabs,seymodel_deabs,0.5,2.0)
        norm210_qso[i] = sptools.integ_spectr(energqso_deabs,qsomodel_deabs,2.0,10.0)
        norm210_sey[i] = sptools.integ_spectr(energsey_deabs,seymodel_deabs,2.0,10.0)
        norm101000_qso[i] = sptools.integ_spectr(energqso_deabs,qsomodel_deabs,10.0,1000.0)
        norm101000_sey[i] = sptools.integ_spectr(energsey_deabs,seymodel_deabs,10.0,1000.0)


    ## Make FITS file
    formatstr = "%iE" % npoints
    col1 = pf.Column(name="gammamin",format='E',array=gammamin)
    col2 = pf.Column(name="gammamax",format='E',array=gammamax)
    col3 = pf.Column(name="gammamed",format='E',array=gammamed)
    col4 = pf.Column(name="nhmin",format='E',array=10.**lognhmin)
    col5 = pf.Column(name="nhmax",format='E',array=10.**lognhmax)
    col6 = pf.Column(name="nhmed",format='E',array=10.**lognhmed)
    # Seyfert extension
    col7a = pf.Column(name="energy",format=formatstr,array=all_e_sey)
    col8a = pf.Column(name="spectrum",format=formatstr,array=all_s_sey)
    col9a = pf.Column(name="deabs_energy",format=formatstr,array=all_e_sey_deabs)
    col10a = pf.Column(name="deabs_spectrum",format=formatstr,array=all_s_sey_deabs)
    col11a = pf.Column(name="norm052",format='E',array=norm052_sey)
    col12a = pf.Column(name="norm210",format='E',array=norm210_sey)
    col13a = pf.Column(name="norm101000",format='E',array=norm101000_sey)
    colarray = [col1,col2,col3,col4,col5,col6,col7a,col8a,col9a,col10a,col11a,col12a,col13a]
    pfcols = pf.ColDefs(colarray)
    tbhdusey = pf.BinTableHDU.from_columns(pfcols)
    # QSO extension
    col7b = pf.Column(name="energy",format=formatstr,array=all_e_qso)
    col8b = pf.Column(name="spectrum",format=formatstr,array=all_s_qso)
    col9b = pf.Column(name="deabs_energy",format=formatstr,array=all_e_qso_deabs)
    col10b = pf.Column(name="deabs_spectrum",format=formatstr,array=all_s_qso_deabs)
    col11b = pf.Column(name="norm052",format='E',array=norm052_qso)
    col12b = pf.Column(name="norm210",format='E',array=norm210_qso)
    col13b = pf.Column(name="norm101000",format='E',array=norm101000_qso)
    colarray = [col1,col2,col3,col4,col5,col6,col7b,col8b,col9b,col10b,col11b,col12a,col13a]
    pfcols = pf.ColDefs(colarray)
    tbhduqso = pf.BinTableHDU.from_columns(pfcols)

    primaire = pf.PrimaryHDU()
    hduarray = [primaire,tbhdusey,tbhduqso]
    thdulist = pf.HDUList(hduarray)
    thdulist.writeto(bankname,clobber=True)















#------------
def create_specbankPL(bankname,dirtemp,gammamin,gammamax,ngammas,npoints=1000):

    """
    AGE: Modified version of Nicolas' scripts. Generates
    spectral bank for PL spectra with a range of Gammas. 
    * bankname: name of FITS file to be created
    * dirtemp : directory where XSpec computations are performed
    * gammamin: minimum value of gamma
    * gammamax: maximum value of gamma
    * ngammas : number of gamma bins
    * npoints : number of points for each spectrum

    Result is a FITS file : 
    Columns corresponding to:
     - gammamin,gammamax,gammamed : bin of gamma
     - energy, spectrum : model spectrum
     - deabs_energy, deabs_spectrum : model for the de-absorbed spectrum (i.e. same powerlaw normalization)
     - norm052, norm210, norm101000 : quick saves of normalization for the de-absorbed spectra in 0.5-2 ; 2-10 ; 10-1000 keV

     Example:
         create_specbank('toto_bank.fits','./tmp_bank/',1.05,2.75,50,1e-20,1e20,1,npoints=1000)
         will create a spectral band "toto_bank.fits" with 50 Gamma's in
         [1.05,2.75] and 1 Nh = 1e-20 (actually will be put artificially to
         logNh=0, as a standard here for unobscured sources (logNh<21).)
    """

    ## Design values
    gtemp = np.linspace(gammamin,gammamax,ngammas+1)
    gmin = gtemp[0:-1]
    gmax = gtemp[1:]

    Nspectra = ngammas

    gammamin = np.resize(gmin,Nspectra)
    gammamax = np.resize(gmax,Nspectra)
    gammamed = 0.5*(gammamin+gammamax)
    gammamed=gtemp

    norm052 = np.zeros(Nspectra)
    norm210 = np.zeros(Nspectra)
    norm101000 = np.zeros(Nspectra)


    all_e = np.zeros((Nspectra,npoints))
    all_s = np.zeros((Nspectra,npoints))
    all_e_deabs = np.zeros((Nspectra,npoints))
    all_s_deabs = np.zeros((Nspectra,npoints))

    ## Create spectra
    for i in range(Nspectra):
        phoindex = gammamed[i]
       
        print " Bank entry: phoindex=%f" % (phoindex)
        energ_deabs, model_deabs=mk_pl(dirtemp,phoindex,npoints=npoints,verbose=0,addstring='bankPL_')

        energ = energ_deabs
        model = model_deabs


        all_e[i,:] = energ
        all_s[i,:] = model
        all_e_deabs[i,:] = energ_deabs
        all_s_deabs[i,:] = model_deabs


        norm052[i] = sptools.integ_spectr(energ_deabs,model_deabs,0.5,2.0)
        norm210[i] = sptools.integ_spectr(energ_deabs,model_deabs,2.0,10.0)
        norm101000[i] = sptools.integ_spectr(energ_deabs,model_deabs,10.0,1000.0)

    
    ## Make FITS file
    formatstr = "%iE" % npoints
    col1 = pf.Column(name="gammamin",format='E',array=gammamin)
    col2 = pf.Column(name="gammamax",format='E',array=gammamax)
    col3 = pf.Column(name="gammamed",format='E',array=gammamed)
    # Seyfert extension
    col7a = pf.Column(name="energy",format=formatstr,array=all_e)
    col8a = pf.Column(name="spectrum",format=formatstr,array=all_s)
    col9a = pf.Column(name="deabs_energy",format=formatstr,array=all_e_deabs)
    col10a = pf.Column(name="deabs_spectrum",format=formatstr,array=all_s_deabs)
    col11a = pf.Column(name="norm052",format='E',array=norm052)
    col12a = pf.Column(name="norm210",format='E',array=norm210)
    col13a = pf.Column(name="norm101000",format='E',array=norm101000)
    colarray = [col1,col2,col3,col7a,col8a,col9a,col10a,col11a,col12a,col13a]
    pfcols = pf.ColDefs(colarray)
    tbhdu = pf.BinTableHDU.from_columns(pfcols)

    primaire = pf.PrimaryHDU()
    hduarray = [primaire,tbhdu]
    thdulist = pf.HDUList(hduarray)
    thdulist.writeto(bankname,clobber=True)
   






#-----------
def read_specbankPL(bankname,phoindex,z,luminosity,bandlum=[0.5,2.0]):
    """
    AGE: Modified version of Nicolas' subroutine to read PL Spectral bank

    Read a spectral bank created with create_specbank
    
    * bankname  : bank to be read
    * phoindex  : array of photon indexes
    * z         : array of redshifts
    * bandlum   : corresponding luminosity
    * QSO_thresh: luminosity for choosing a QSO rather that a Sey model
    
    Result is 6 lists: one for the energy arrays, the other for
    the spectral arrays, one list consisting of the type (sey,qso).
    The latter three correspond to the 0.5-2 ; 2-10 ; 10-1000 normalizations
    of the corresponding De-absorbed spectrum.


    Warning: No normalization of spectra according to luminosity
    (must be done afterwards through the provided normalization factors)
    """
    
    print " > opening spectral Bank: %s" % bankname
    ## Open bank file
    data = pf.open(bankname)[1].data


    ## Check array sizes
    npho = np.size(phoindex)
    nz   = np.size(z)
    nlum = np.size(luminosity)
    
    nspec = max([npho,nz,nlum])
    if(npho == 1):
        phovec = np.zeros(nspec)+phoindex
    elif(npho != nspec):
        print "Size mismatch (phoindex)"
        sys.exit(1)
    else:
        phovec = phoindex.copy()
        

    if(nz == 1):
        zvec = np.zeros(nspec)+z
    elif(nz != nspec):
        print "Size mismatch (z)"
        sys.exit(1)
    else:
        zvec = z.copy()

    if(nlum == 1):
        lumvec = np.zeros(nspec)+luminosity
    elif(nlum != nspec):
        print "Size mismatch (luminosity)"
        sys.exit(1)
    else:
        lumvec = luminosity.copy()


    ## Loop over each spectrum
    energies = []
    spectra  = []
    goodlist = []
    norm052list = []
    norm210list = []
    norm101000list = []


    for i in range(nspec):

	# Apply condition
        cond = (phovec[i] >= data.field("gammamin")) * (phovec[i] < data.field("gammamax"))

        print phovec[i] 
        #print data.field("gammamin")
        #print data.field("gammamin") 

        
        temp, = np.where(cond)
        pos = temp[0]

        print "For Gamma=",phovec[i]," the database positions is",pos
        
       	# Check unicity
        if(np.size(pos) != 1):
            print "Seyfert bank problem"
            print " Gamma=%f" % phovec[i]
            print " Nh=%f" % nhvec[i]
            sys.exit(1)

        energ  = data.field("energy")[pos][:]
        model = data.field("spectrum")[pos][:]
        energ_deabs = data.field("deabs_energy")[pos][:]
        model_deabs = data.field("deabs_spectrum")[pos][:]

        # Select QSO/Seyfert
        norm_052 = data.field("norm052")[pos]
       
        norm_bnd = sptools.integ_spectr(energ_deabs,model_deabs,bandlum[0],bandlum[1])


        lum_052 = lumvec[i]*(norm_052/norm_bnd)

        energ,spectrum = energ.copy(),model.copy()
        deabs_energ,deabs_spectrum = energ_deabs.copy(),model_deabs.copy()

        good="sey"
        

        e,s =  sptools.redsh_spectr(energ,spectrum,zvec[i])
        deabs_e,deabs_s = sptools.redsh_spectr(deabs_energ,deabs_spectrum,zvec[i])
        # Compute rest-frames norms (!! redsh_spectr applies an arbitrary normalization !!)
        #   on intrinsic spectra
        n052 = sptools.integ_spectr(deabs_e,deabs_s,0.5/(1.+zvec[i]),2.0/(1.+zvec[i]))
        n210 = sptools.integ_spectr(deabs_e,deabs_s,2.0/(1.+zvec[i]),10.0/(1.+zvec[i]))
        n101000 = sptools.integ_spectr(deabs_e,deabs_s,10.0/(1.+zvec[i]),1000.0/(1.+zvec[i]))

        energies.append(e)
        spectra.append(s)
        goodlist.append(good)
        norm052list.append(n052)
        norm210list.append(n210)
        norm101000list.append(n101000)

    return energies,spectra,goodlist,norm052list,norm210list,norm101000list


