"""
This is a class replicating the SIMPUT format for Python
 * contains a initializer
 * a writer

For the moment, cannot handle spectra with different sizes.
Also please check the upper case, not automatically done.
Equinox, RADESYS hard coded.
Should have only 1 SRC_CAT, 1 SPECTRUM and 1 IMAGE extension

Author: N.Clerc
Created: 20/04/2012

Updated: 22.11.2012, for inclusion of PHLIST
         26.02.2014, for inclusion of LCURVE (*not fully tested yet*)
         12.01.2016, for compatibility with Pyfits v3.3

"""

import pyfits as pf
#import astropy.io.fits as pf
import numpy as np
import sys
import string as st

#=======================================================================================================
# ---------------------------------------------
# ---------------------------------------------
#=======================================================================================================


class simputcatalogue:
    
    # ~~~~~~~~~~~~~~ Method to initialize standard SRC_CAT extension ~~
    def __init__(self,colnames=[],units={}):
        
        # Initialize header
        header = {"HDUCLASS":'HEASARC',
                  "HDUCLAS1":'SIMPUT',
                  "HDUVERS":'1.0.0',
                  "HDUCLAS2":'SRC_CAT',
                  "EXTNAME":'SRC_CAT',
                  "RADESYS":'FK5',
                  "EQUINOX":2000.}
        self.header=header

        # Initialize data (mandatory fields+user columns)
        self.data = {"SRC_ID":np.array([],dtype='int'),
                     "RA":np.array([],dtype='float'),"DEC":np.array([],dtype='float'),
                     "E_MIN":np.array([],dtype='float'),"E_MAX":np.array([],dtype='float'),
                     "FLUX":np.array([],dtype='float'),
                     "SPECTRUM":np.array([],dtype='|S256'),
                     "IMAGE":np.array([],dtype='|S256'),
                     "TIMING":np.array([],dtype='|S256')}
        user_columns={}
        for col in colnames:
            if(col in self.data.keys()):
                print "** Forbidden: '%s' column is already in the extension" % col
                print "   (Action: quit)"
                print
                sys.exit(1)
            else: user_columns[col] = np.array([])
        self.data.update(user_columns)

        # Set units
        self.units = {"RA":'deg',"DEC":'deg',"IMGROTA":'deg',
                      "E_MIN":'keV',"E_MAX":'keV',
                      "FLUX":'erg/s/cm**2'}
        self.units.update(units)

        # Other content
        self.type = 'catalogue'
        self.nsources = 0




    # ~~~~~~~~~~~~~~ Method to add 1 source ~~
    def addSource(self,srcdict):
        if(srcdict["SRC_ID"] in self.data["SRC_ID"]):
            print "** addSource: already one source with this src_id"
            print "   (Action: do not add source)"
            print
            return None

        thesekeys = srcdict.keys()
        didit=False
        for key in self.data.keys():
            if(not key in thesekeys):
                print "** addSource: missing key '%s'" % key
                print "**            (Action: abort)"
                print
                sys.exit(1)

        for key in thesekeys:
            if(not (key in self.data.keys())):
                   print "** addSource: no such field in current data: '%s'" % key
                   print "**            (Action: Neglecting field)"
                   print
                   continue
            self.data[key] = np.append(self.data[key],srcdict[key])
            didit=True

        if(didit):self.nsources+=1
        return None

    
    # ~~~~~~~~~~~~~~~~ Method to get one source ~~
    def getSource(self,src_id):
        if(self.nsources == 0):
            print "** getSource: empty catalogue"
            print
            return None
        if(not src_id in self.data["SRC_ID"]):
            print "** getSource: no such SRC_ID: %i" % src_id
            print
            return None
        else:
            result = {}
            (position,) = np.where(self.data["SRC_ID"] == src_id)
            position = position[0]
            for key in self.data.keys():                
                result[key] = self.data[key][position]

            return result


    #~~~~~~~~~~~~~~~~~~ Method to create PyFits HDU ~~
    def toHDU(self):
        listorder = ['SRC_ID','SRC_NAME','RA','DEC','IMGROTA','IMGSCAL',
                     'E_MIN','E_MAX','FLUX','SPECTRUM','IMAGE','TIMING']
        typesorder = ['K','A','E','E','E','E',
                      'E','E','E','A','A','A']
        applysize=[False,True,False,False,False,False,
                   False,False,False,True,True,True] #Which column have a length "40A"...
        # Gathers columns
        colarray = []
        hdrunits = {}
        colnumero=1 #starts at 1 in FITS format
        for i in range(len(listorder)):
            keylow = map(lambda n: n.upper(), self.data.keys())
            item=listorder[i].upper()
            if(item in keylow):
                if(applysize[i]): 
                    maxsize=max(map(lambda n: len(n), self.data[item]))
                    maxstr='%i' % maxsize
                    thistype=maxstr+typesorder[i]
                else: thistype=typesorder[i]
                mycol=pf.Column(name=item, format=thistype, 
                                array=self.data[item])
                colarray.append(mycol)
                if(item in self.units.keys()):
                    hdrunits['TUNIT%i' % colnumero] = self.units[item]
                colnumero += 1
        
        pfcols = pf.ColDefs(colarray)
        #tbhdu = pf.new_table(pfcols) # Deprecated...
        tbhdu = pf.BinTableHDU.from_columns(pfcols)

        # The header
        for card in self.header.keys():
            #tbhdu.header.update(card,self.header[card]) # Deprecated
            tbhdu.header[card] = self.header[card]
        for card in hdrunits.keys():
            #tbhdu.header.update(card,hdrunits[card]) # Deprecated
            tbhdu.header[card] = hdrunits[card]

        return tbhdu

#=======================================================================================================
# ---------------------------------------------
# ---------------------------------------------
#=======================================================================================================


class simputspectrum:

    #~~~~~~~~~~~~~~~~~~~ Method to initialize standard SPECTRUM extension ~~
    def __init__(self,speclength,colnames=[],units={}):
        
        # Initialize header
        header = {"HDUCLASS":'HEASARC',
                  "HDUCLAS1":'SIMPUT',
                  "HDUVERS":'1.0.0',
                  "HDUCLAS2":'SPECTRUM',
                  "EXTNAME":'SPECTRUM'}
        self.header=header
        
        # Initialize data (mandatory fields+user columns)
        self.data = {"ENERGY":np.transpose(np.array([[]]*speclength,dtype='float64')),
                     "FLUX":  np.transpose(np.array([[]]*speclength,dtype='float64'))}
        user_columns={}
        for col in colnames:
            if(col in self.data.keys()):
                print "** Forbidden: '%s' column is already in the extension" % col
                print "   (Action: quit)"
                print
                sys.exit(1)
            else: user_columns[col] = np.array([])
        self.data.update(user_columns)

        # Set units
        self.units = {"ENERGY":'keV',"FLUX":'photons/s/cm**2/keV'}
        self.units.update(units)
        
        # Other content
        self.type = 'spectrum'
        self.nspectra = 0
        self.speclength = speclength


    #~~~~~~~~~~~~~~~~~~~~ Method to add a spectrum to the list ~~
    def addSpectrum(self,newspectrum):
        thesekeys = newspectrum.keys()
        didit=False
        for key in self.data.keys():
            if(not key in thesekeys):
                print "** addSource: missing key '%s'" % key
                print "**            (Action: abort)"
                print
                sys.exit(1)
        for key in thesekeys:
            if(not (key in self.data.keys())):
                print "** addSpectrum: no such field in current data: '%s'" % key
                print "                (Action: Neglecting field)"
                print
                continue
            if(key == 'ENERGY' or key == 'FLUX'):
                ## Here we are dealing with (nspectra,speclength) shapes
                self.data[key] = np.vstack([self.data[key],newspectrum[key]])
            else:
                ## Here we are dealing with (nspectra) shape
                self.data[key] = np.append(self.data[key],newspectrum[key])
            didit=True
                
        if(didit):self.nspectra+=1
        return None


    #~~~~~~~~~~~~~~~~~~~~~~ Method to get a spectrum from the list ~~
    def getSpectrum(self,row):
        if(self.nspectra == 0):
            print "** getSpectrum: empty catalogue"
            print
            return None
        if(row == 0):
            print "** getSpectrum: row indexing begins at 1"
            print
            return None
        if(row > self.nspectra):
            print "** getSpectrum: no such row: %i" % row
            print
            return None
        else:
            result = {"ENERGY":np.zeros(self.speclength),"FLUX":np.zeros(self.speclength)}
            for key in self.data.keys():
                if(key == 'ENERGY' or key=='FLUX'):
                    result[key] = self.data[key][row-1,:].copy()
                else:
                    result[key] = self.data[key][row-1]

            return result


    # ~~~~~~~~~~~~~~~~~~~~~~~ Method to create PyFits HDU ~~
    def toHDU(self):
        tablength = "%i" % (self.speclength)
        listorder = ['NAME','ENERGY','FLUX']
        typesorder = ['A',tablength+'E',tablength+'E']
        applysize=[True,False,False]

        #Gathers columns
        colarray = []
        hdrunits = {}
        colnumero = 1
        for i in range(len(listorder)):
            keylow = map(lambda n: n.upper(), self.data.keys())
            item = listorder[i].upper()
            if(item in keylow):
                if(applysize[i]): 
                    maxsize=max(map(lambda n: len(n), self.data[item]))
                    maxstr='%i' % maxsize
                    thistype=maxstr+typesorder[i]
                else: thistype=typesorder[i]
                mycol=pf.Column(name=item, format=thistype,
                                array=self.data[item])
                colarray.append(mycol)
                if(item in self.units.keys()):
                    hdrunits['TUNIT%i' % colnumero] = self.units[item]
                colnumero += 1

        pfcols = pf.ColDefs(colarray)
        #tbhdu = pf.new_table(pfcols) # Deprecated
        tbhdu = pf.BinTableHDU.from_columns(pfcols)

        # The header
        for card in self.header.keys():
            #tbhdu.header.update(card,self.header[card]) # Deprecated
            tbhdu.header[card] = self.header[card]
        for card in hdrunits.keys():
            #tbhdu.header.update(card,hdrunits[card]) # Depredcated
            tbhdu.header[card] = hdrunits[card]

        return tbhdu        

#=======================================================================================================
# ---------------------------------------------
# ---------------------------------------------
#=======================================================================================================


class simputimage:
    def __init__(self):
        header = {"HDUCLASS":'HEASARC',
                  "HDUCLAS1":'SIMPUT',
                  "HDUVERS":'1.0.0',
                  "HDUCLAS2":'IMAGE',
                  "EXTNAME":'IMAGE'}

        self.data={}
        self.header=header
        self.type = 'image'

    def toHDU(self):
        pass

#=======================================================================================================
# ---------------------------------------------
# ---------------------------------------------
#=======================================================================================================


class simputphlist:
    def __init__(self):
        header = {"HDUCLASS":'HEASARC',
                  "HDUCLAS1":'SIMPUT',
                  "HDUVERS":'1.0.0',
                  "HDUCLAS2":'PHOTONS',
                  "EXTNAME":'PHLIST',
                  "REFRA":0.0,
                  "REFDEC":0.0}

        self.data={}
        self.header=header
        self.type = 'phlist'

        # Initialize data (mandatory fields+user columns)
        self.data = {"ENERGY":np.array([],dtype='int'),
                     "RA":np.array([],dtype='float'),
                     "DEC":np.array([],dtype='float')}
        
        # Set units
        self.units = {"ENERGY":'keV',"RA":'deg',"DEC":'deg'}

    #~~~~~~~~~~~~~~~~~~ Method to add a photon list
    def addPhotonList(self,phlistdict):
        self.data["ENERGY"] = np.append(self.data["ENERGY"],phlistdict["ENERGY"])
        self.data["RA"] = np.append(self.data["RA"],phlistdict["RA"])
        self.data["DEC"] = np.append(self.data["DEC"],phlistdict["DEC"])
        return None


    #~~~~~~~~~~~~~~~~~~ Method to create PyFits HDU ~~
    def toHDU(self):
        listorder = ['ENERGY','RA','DEC']
        typesorder = ['E','E','E']
        
        # Gathers columns
        colarray = []
        hdrunits = {}
        colnumero=1 #starts at 1 in FITS format
        for i in range(len(listorder)):
            keylow = map(lambda n: n.upper(), self.data.keys())
            item=listorder[i].upper()
            if(item in keylow):
                thistype=typesorder[i]
                mycol=pf.Column(name=item, format=thistype, 
                                array=self.data[item])
                colarray.append(mycol)
                if(item in self.units.keys()):
                    hdrunits['TUNIT%i' % colnumero] = self.units[item]
                colnumero += 1

        pfcols = pf.ColDefs(colarray)
        tbhdu = pf.new_table(pfcols)

        # The header
        for card in self.header.keys():
            #tbhdu.header.update(card,self.header[card]) # Deprecated
            tbhdu.header[card] = self.header[card]
        for card in hdrunits.keys():
            #tbhdu.header.update(card,hdrunits[card]) # Deprecated
            tbhdu.header[card] = hdrunits[card]

        return tbhdu


#=======================================================================================================
# ---------------------------------------------
# ---------------------------------------------
#=======================================================================================================


class simputlcurve:

    #~~~~~~~~~~~~~~~~~~~ Method to initialize standard LCURVE extension ~~
    def __init__(self,colnames=[],units={},options={}):


        # Initialize header
        header = {"HDUCLASS":'HEASARC',
                  "HDUCLAS1":'SIMPUT',
                  "HDUVERS":'1.0.0',
                  "HDUCLAS2":'LIGHTCUR',
                  "EXTNAME":'LIGHTCUR',
                  "MJDREF":options["MJDREF"],
                  "TSTART":options["TSTART"],
                  "TSTOP":options["TSTOP"],
                  "TIMEZERO":options["TIMEZERO"],
                  "TIMESYS":options["TIMESYS"],
                  "TIMEUNIT":options["TIMEUNIT"],
                  "CLOCKCOR":options["CLOCKCOR"],
                  "PERIODIC":options["PERIODIC"]
                  }
        self.header=header
        
        if(options["PERIODIC"] == 1):
            temp = {"PHASE0":options["PHASE0"],"PERIOD":options["PERIOD"]}
            self.header.update(temp)

        # Initialize data (mandatory fields+user columns)
        self.data = {"FLUX": np.array([],dtype='float64')}
        user_columns={}
        for col in colnames:
            if(col in self.data.keys()):
                print "** Forbidden: '%s' column is already in the extension" % col
                print "   (Action: quit)"
                print
                sys.exit(1)
            else: user_columns[col] = np.array([])
        self.data.update(user_columns)

        # Test whether TIME or PHASE exist
        if( (not "TIME" in self.data.keys()) and (not "PHASE" in self.data.keys())):
            print "** Error: either TIME or PHASE must exist"
            print "       (Action: quit)"
            print
            sys.exit(1)

        # Set units
        self.units = {}
        self.units.update(units)

        # Other content
        self.type = 'lcurve'
        self.ntimes = 0

    #~~~~~~~~~~~~~~~~~~~ Method to add a timing event  ~~
    def addTiming(self,srcdict):

        datakeys = self.data.keys()
        if("TIME" in datakeys):
            if(srcdict["TIME"] in self.data["TIME"]):
                print "** addTiming: already one entry with this TIME"
                print "   (Action: do not add row)"
                print
                return None
        elif("PHASE" in datakeys):
            if(srcdict["PHASE"] in self.data["PHASE"]):
                print "** addSource: already one entry with this PHASE"
                print "   (Action: do not add row)"
                print
                return None

        thesekeys = srcdict.keys()
        didit=False
        for key in self.data.keys():
            if(not key in thesekeys):
                print "** addTiming: missing key '%s'" % key
                print "**            (Action: abort)"
                print
                sys.exit(1)

        for key in thesekeys:
            if(not (key in self.data.keys())):
                   print "** addTiming: no such field in current data: '%s'" % key
                   print "**            (Action: Neglecting field)"
                   print
                   continue
            self.data[key] = np.append(self.data[key],srcdict[key])
            didit=True

        if(didit):self.ntimes += 1
        return None

    # ~~~~~~~~~~~~~~~~ Method to get one time ~~
    def getTime(self,time):
        pass

    #~~~~~~~~~~~~~~~~~~ Method to create PyFits HDU ~~
    def toHDU(self):
        listorder = ['TIME','PHASE','FLUX','SPECTRUM','IMAGE']
        typesorder = ['E','E','E','A','A']
        applysize = [False,False,False,True,True]

        # Gathers columns
        colarray = []
        hdrunits = {}
        colnumero=1 #starts at 1 in FITS format
        for i in range(len(listorder)):
            keylow = map(lambda n: n.upper(), self.data.keys())
            item=listorder[i].upper()
            if(item in keylow):
                if(applysize[i]): 
                    maxsize=max(map(lambda n: len(n), self.data[item]))
                    maxstr='%i' % maxsize
                    thistype=maxstr+typesorder[i]
                else: thistype=typesorder[i]
                mycol=pf.Column(name=item, format=thistype, 
                                array=self.data[item])
                colarray.append(mycol)
                if(item in self.units.keys()):
                    hdrunits['TUNIT%i' % colnumero] = self.units[item]
                colnumero += 1

        pfcols = pf.ColDefs(colarray)
        tbhdu = pf.new_table(pfcols)

        # The header
        for card in self.header.keys():
            #tbhdu.header.update(card,self.header[card]) # Deprecated
            tbhdu.header[card] = self.header[card]
        for card in hdrunits.keys():
            #tbhdu.header.update(card,hdrunits[card]) # Deprecated
            tbhdu.header[card] = hdrunits[card]

        return tbhdu


#=======================================================================================================
#------------------------------------------
#------------------------------------------
#=======================================================================================================


class simputobj:

    # -- Constructor --
    def __init__(self,build_extensions=[],cat_columns=[],cat_units={},
                 spec_columns=[],spec_units={},lc_columns=[],lc_units={},lc_options={},speclength=200):
        
        if('catalogue' in build_extensions):
            self.catalogue = simputcatalogue(colnames=cat_columns,units=cat_units)
        if('spectrum' in build_extensions):
            self.spectrum = simputspectrum(speclength,colnames=spec_columns,units=spec_units)
        if('image' in build_extensions):
            self.image = simputimage()
        if('lcurve' in build_extensions):
            mylc_options = {}
            allopt = {'MJDREF':51544.0,'TSTART':0.0,'TSTOP':0.0,'TIMEZERO':0.0,'TIMESYS':'MJD',
                      'TIMEUNIT':'s','CLOCKCOR':'YES','PERIODIC':1,'PHASE0':0.0,'PERIOD':0.0}
            for opt in allopt.keys():
                if(opt in lc_options.keys()):
                    mylc_options[opt] = lc_options[opt]
                else:
                    mylc_options[opt] = allopt[opt]

            self.lcurve = simputlcurve(colnames=lc_columns,units=lc_units,options=mylc_options)
        if('phlist' in build_extensions):
            self.phlist = simputphlist()

        self.extensions=build_extensions[:] ## Copy otherwise propagates everywhere !!

    # -- Test existence of extensions --
    def hasextension(self,extension,verbose=0):
        val = (extension in self.extensions)
        if(verbose >= 1): print "--> (%s)" % (val)
        return val
    

    # -- Add one void extension to object --
    def addextension(self,extname,cat_columns=[],cat_units={},
                     spec_columns=[],spec_units={},lc_columns=[],lc_units={},
                     lc_options={},speclength=200):
        if(self.hasextension(extname)):
            print "** simputobj.addextension: object already has extension %s" % extname
            print "**                         (Action: neglect command)"
            print
        else:
            if(extname == 'catalogue'):
                self.catalogue = simputcatalogue(colnames=cat_columns,units=cat_units)
            if(extname == 'spectrum'):
                self.spectrum = simputspectrum(speclength,colnames=spec_columns,units=spec_units)
            if(extname == 'image'):
                self.image = simputimage()
            if(extname == 'phlist'):
                self.phlist = simputphlist()
            if(extname == 'lcurve'):
                self.lcurve = simputlcurve(colnames=lc_columns,units=lc_units,options=lc_options)
            self.extensions.append(extname)


    # -- Delete one extension of object --
    def delextension(self,extension):
        if(not self.hasextension(extension)):
            print "** simputobj.delextension: no such extension"
            print
        else:
            tempext = filter(lambda ext: ext != extension,self.extensions)
            self.extensions = tempext[:] ## Make a copy!
            return delattr(self,extension)


    # -- Write into a FITS file --
    def writeto(self,filename,clobber=True):
        primaire = pf.PrimaryHDU()
        hduarray = [primaire]
        # Loop over extensions
        for ext in self.extensions:
            thisext = getattr(self,ext)
            tbhdu=thisext.toHDU()
            hduarray.append(tbhdu)
        # Gathers extensions and write to file
        thdulist = pf.HDUList(hduarray)
        thdulist.writeto(filename,clobber=clobber)


#=======================================================================================================
# ---------------------------------------------
# ---------------------------------------------
#=======================================================================================================

def readfile(filename):
    
    print "Reading SIMPUT file %s ..." % filename
    # Open with Pyfits
    tbhdulist=pf.open(filename)
    
    # Find existing extensions
    fullliste =['SRC_CAT','SPECTRUM','IMAGE','PHLIST','LIGHTCUR']
    maliste=[]
    notfound=[]

    for ext in fullliste:
        try:
            idx = tbhdulist.index_of(ext)
            maliste.append(ext)
        except KeyError:
            notfound.append(ext)
    
    print "  Found extensions: ",maliste
    print "  (not present in file: ",notfound,")"
    
    if(len(maliste) == 0):
        print "** readfile: no valid extension found"
        print " ** Is it a SIMPUT FITS file?"
        sys.exit(1)
    
    result = simputobj()

    # 1/ Construct catalogue extension
    if('SRC_CAT' in maliste):
        hdu=tbhdulist['SRC_CAT']
        hdr=hdu.header
        data=hdu.data
        cols=hdu.columns
        
        ## Build units
        my_units={}
        for cc in range(len(cols.names)):
            uni=cols.units[cc]
            nam=cols.names[cc].upper()
            if(nam != ''):
                my_units[nam.upper()]=uni

        ## Build optional columns
        required=['SRC_ID','RA','DEC','E_MIN','E_MAX','FLUX','SPECTRUM','IMAGE','TIMING']
        my_columns=[]
        for nam in cols.names:
            naml = nam.upper()
            if(naml in required):continue
            my_columns.append(naml)

        ## Construct extension
        result.addextension('catalogue',cat_columns=my_columns,cat_units=my_units)
        
        ## Fill in the data (copy column by column > faster)
        for nam in cols.names:
            naml=nam.upper()
            result.catalogue.data[naml] = data.field(naml).copy()
            result.catalogue.nsources=len(data)
        
        print "  Detected: %i sources" % len(data)


    # 2/ Construct spectrum extension
    if('SPECTRUM' in maliste):
        hdu=tbhdulist['SPECTRUM']
        hdr=hdu.header
        data=hdu.data
        cols=hdu.columns
        
        ## Build units
        my_units={}
        for cc in range(len(cols.names)):
            uni=cols.units[cc]
            nam=cols.names[cc].upper()
            if(nam != ''):
                my_units[nam.upper()]=uni

        ## Build optional columns
        required=['ENERGY','FLUX']
        my_columns=[]
        for nam in cols.names:
            naml = nam.upper()
            if(naml in required):continue
            my_columns.append(naml)

        ## Get Speclength
        tab1 = data.field('ENERGY')
        tab2 = data.field('FLUX')
        len1 = tab1.shape[1]
        len2 = tab2.shape[1]
        if(len1 != len2):
            print "** readfile: size mismatch"
            print
            sys.exit(1)
        speclength=len1
        
        print "  Detected: %i spectra" % len(data)
        print "  Detected speclength: %i elements" % speclength

        ## Construct extension
        result.addextension('spectrum',speclength=speclength,
                            spec_columns=my_columns,
                            spec_units=my_units)
        
        ## Fill in the data (copy column by column > faster)
        for nam in cols.names:
            naml=nam.upper()
            result.spectrum.data[naml] = data.field(naml).copy()
            result.spectrum.nspectra=len(data)

    # 3/ Construct phlist extension
    if('PHLIST' in maliste):
        hdu=tbhdulist['PHLIST']
        hdr=hdu.header
        data=hdu.data
        cols=hdu.columns
        
        ## Build units
        my_units={}
        for cc in range(len(cols.names)):
            uni=cols.units[cc]
            nam=cols.names[cc].upper()
            if(nam != ''):
                my_units[nam.upper()]=uni

        ## Build optional columns
        required=['ENERGY','RA','DEC']
        my_columns=[]
        for nam in cols.names:
            naml = nam.upper()
            if(naml in required):continue
            my_columns.append(naml)

        ## Construct extension
        result.addextension('phlist')
        
        ## Fill in the data (copy column by column > faster)
        for nam in cols.names:
            naml=nam.upper()
            result.phlist.data[naml] = data.field(naml).copy()
            result.phlist.nsources=len(data)
        
        print "  Detected: %i photons" % len(data)


    # 4/ Construct lcurve extension
    if('LIGHTCUR' in maliste):
        hdu=tbhdulist['LIGHTCUR']
        hdr=hdu.header
        data=hdu.data
        cols=hdu.columns
        
        ## Build units
        my_units={}
        for cc in range(len(cols.names)):
            uni=cols.units[cc]
            nam=cols.names[cc].upper()
            if(nam != ''):
                my_units[nam.upper()]=uni

        ## Build optional columns
        required=['FLUX']
        my_columns=[]
        for nam in cols.names:
            naml = nam.upper()
            if(naml in required):continue
            my_columns.append(naml)

        ## Build option vector
        allopt = {'MJDREF':51544.0,'TSTART':0.0,'TSTOP':0.0,'TIMEZERO':0.0,'TIMESYS':'MJD',
                  'TIMEUNIT':'s','CLOCKCOR':'YES','PERIODIC':1,'PHASE0':0.0,'PERIOD':0.0}
        my_options = {}
        for optname in allopt.keys():
            if(optname in hdr.keys()):
                my_options[optname] = hdr[optname]
            else:
                my_options[optname] = allopt[optname]

        ## Construct extension
        result.addextension('lcurve',lc_columns=my_columns,lc_units=my_units,lc_options=my_options)
        
        ## Fill in the data (copy column by column > faster)
        for nam in cols.names:
            naml=nam.upper()
            result.lcurve.data[naml] = data.field(naml).copy()
            result.lcurve.nsources=len(data)
        
        print "  Detected: %i times" % len(data)




    print
    return result








