#! /usr/bin/env python

import simput
import os
import sys
import numpy as np
import numpy.core.defchararray as npstr
import string
from math import pi
import random

def mydist_on_sphere(l1,b1,l2,b2):
#-----------------------------------------------------------
   """
Formula for distance on sphere accurate over entire sphere
(Vincenty, Thaddeus, 1975). Input and output are in degrees.

:param l1:
   Longitude of first location on sphere
:type l1:
   float
:param b1:
   Latitude of first location on sphere
:type b1:
   float
:param l2:
   Longitude of second location on sphere
:type l2:
   float
:param b2:
   Latitude of second location on sphere
:type b2:
   float

:Examples:

   >>> from kapteyn.maputils import dist_on_sphere
   >>> print dist_on_sphere(0,0, 20,0)
       20.0

   >>> print dist_on_sphere(0,30, 20,30)
       17.2983302106

   """
#-----------------------------------------------------------
   fac = np.pi / 180.0
   l1 *= fac; b1 *= fac; l2 *= fac; b2 *= fac
   dlon = l2 - l1
   aa1 = np.cos(b2)*np.sin(dlon)
   cosdlon = np.cos(dlon)
   dlon = 0

   aa2 = np.cos(b1)*np.sin(b2) - np.sin(b1)*np.cos(b2)*cosdlon
   a = np.sqrt(aa1*aa1+aa2*aa2)
   aa1 = 0
   aa2 = 0

   b = np.sin(b1)*np.sin(b2) + np.cos(b1)*np.cos(b2)*cosdlon
   cosdlon = 0

   d = np.arctan2(a,b)
   a = 0
   b = 0
   return d/fac

# =========================================

def toreg(infile,outfile,size='20"',fluxmin=1e-18,fluxmax=1e-10):
    """
    Converts a simput file to a ds9 region file.

    Author: N.Clerc
    Created: 27/04/2012

    """
    catsim = simput.readfile(infile)

    catalogue = catsim.catalogue
    sources_id = catalogue.data["SRC_ID"]

    lines = []
    lines.append('# Region file created by simputtools.toreg')
    lines.append('# Input file: %s' % infile)
    
    for i in sources_id:
        src = catalogue.getSource(i)
        if( (src["FLUX"] >= fluxmin)*(src["FLUX"] < fluxmax)):
            test = 'fk5; circle(%f, %f, %s) # color=magenta' % (src["RA"],src["DEC"],size)
            lines.append(test)



    outFile = file(outfile,'w')
    text = map(lambda x: '%s\n' % x,lines)
    outFile.writelines(text)
    outFile.close()


#====================================================
def merge(file1,file2,outfile,pref1='A_',pref2='B_',fast=True):


    """
    Merges two SIMPUT files into one
    
    v0.1 : pas de checks des headers
           referencement aux spectres: - si donne par l'ID, alors ca va pas marcher pour le catalogue 2
                                       - si donne par le nom, alors probleme si des spectres du 1 ont 
                                         le meme nom que des spectres du 2
           
    Author : NC, 19.07.2012

    """


    sim1 = simput.readfile(file1)
    sim2 = simput.readfile(file2)

    REF_CAT  = ["SRC_ID","RA","DEC","E_MIN","E_MAX","FLUX","SPECTRUM","IMAGE","TIMING"]
    REF_SPEC = ["ENERGY","FLUX"]


    doext = []
    cat_col = []
    cat_units = {}
    spec_col = []
    spec_units = {}
    speclength = 0
    cat_merge = 0
    spec_merge = 0


    #------------------------------ -- SETUP -- ---------------------------------
    
    #~~ CATALOGUE ~~
    if(sim1.hasextension('catalogue') and sim2.hasextension('catalogue')):
        doext.append('catalogue')
        
        col1 = sim1.catalogue.data.keys()
        col2 = sim2.catalogue.data.keys()        

        if(sorted(col1) != sorted(col2)):
            print "Bad match in catalogue columns"
            print "Cat 1",col1
            print "Cat 2",col2
            sys.exit(1)
        
        if(sim1.catalogue.units != sim2.catalogue.units):
            print "Bad match in catalogue units"
            sys.exit(1)
        
        catmerge = 3
        cat_col = list(set(col1)-set(REF_CAT)) # Keys in col1 not in REF_CAT
        cat_units = sim1.catalogue.units

    elif(sim1.hasextension('catalogue')):
        col1 = sim1.catalogue.keys()
        doext.append('catalogue')
        catmerge = 1
        cat_col = list(set(col1)-set(REF_CAT)) # Keys in col1 not in REF_CAT
        cat_units = sim1.catalogue.units

    elif(sim2.hasextension('catalogue')):
        col2 = sim2.catalogue.keys()        
        doext.append('catalogue')
        catmerge = 2
        cat_col = list(set(col2)-set(REF_CAT)) # Keys in col2 not in REF_CAT
        cat_units = sim2.catalogue.units


    #~~ SPECTRUM ~~
    if(sim1.hasextension('spectrum') and sim2.hasextension('spectrum')):
        doext.append('spectrum')
        
        col1 = sim1.spectrum.data.keys()
        col2 = sim2.spectrum.data.keys()
        
        if(sorted(col1) != sorted(col2)):
            print "Bad match in spectrum columns"
            sys.exit(1)

        if(sim1.spectrum.speclength != sim2.spectrum.speclength):
            print "Bad match in spectrum speclength"
            sys.exit(1)
        
        if(sim1.spectrum.units != sim2.spectrum.units):
            print "Bad match in spectrum units"
            sys.exit(1)

        specmerge = 3
        speclength = sim1.spectrum.speclength
        spec_col = list(set(col1)-set(REF_SPEC)) # Keys in col1 not in REF_SPEC
        spec_units = sim1.spectrum.units

    elif(sim1.hasextension('spectrum')):
        col1 = sim1.spectrum.keys()
        doext.append('spectrum')
        specmerge = 1
        speclength = sim1.spectrum.speclength
        spec_col = list(set(col1)-set(REF_SPEC)) # Keys in col1 not in REF_SPEC
        spec_units = sim1.spectrum.units

    elif(sim2.hasextension('spectrum')):
        col2 = sim2.spectrum.keys()
        doext.append('spectrum')
        specmerge = 2
        speclength = sim2.spectrum.speclength
        spec_col = list(set(col2)-set(REF_SPEC)) # Keys in col2 not in REF_SPEC
        spec_units = sim2.spectrum.units
        

    #------------------------------------------- -- DO IT -- -------------------------------

    # Initialize object
    res = simput.simputobj(doext,cat_columns=cat_col,cat_units=cat_units,spec_columns=spec_col,spec_units=spec_units,speclength=speclength)

    ### .... selon les cas cid-essuis, creer le SIMPUT correspondant...

    ##
    # catmerge = 0 : pas de catalogue
    #          = 1 : seulement sim1
    #          = 2 : seulement sim2
    #          = 3 : sim1+sim2
    #
    # specmerge = 0 : pas de spec
    #           = 1 : seulement sim1
    #           = 2 : seulement sim2
    #           = 3 : sim1+sim2

    print "  > catmerge  = %i" % catmerge
    print "    Facultative Columns: ",cat_col
    print "  > specmerge = %i" % specmerge
    print "    Facultative Columns: ",spec_col

    if(catmerge == 1):
        print " Catalogue 1..."
        res.catalogue.data = sim1.catalogue.data
        res.catalogue.nsources = sim1.catalogue.nsources
    if(catmerge == 2):
        print " Catalogue 2..."
        res.catalogue.data = sim2.catalogue.data
        res.catalogue.nsources = sim2.catalogue.nsources
    if(catmerge == 3):
        id_max = max(sim1.catalogue.data["SRC_ID"])+1

        if(fast):
            print " Fast Mode enabled (Catalogue 1 &2 merging)"
            res.catalogue.nsources = sim1.catalogue.nsources + sim2.catalogue.nsources
            for key in sim1.catalogue.data.keys():
                if(key == "SRC_NAME"):
                    res.catalogue.data[key] = np.append(npstr.add(pref1,sim1.catalogue.data[key]),npstr.add(pref2,sim2.catalogue.data[key]))
                elif(key == "SRC_ID"):
                    res.catalogue.data[key] = np.append(sim1.catalogue.data[key],id_max+sim2.catalogue.data[key])
                else:
                    res.catalogue.data[key] = np.append(sim1.catalogue.data[key],sim2.catalogue.data[key])
        else:
            print " Catalogue 1..."
            for src in range(sim1.catalogue.nsources):
                this_source = {}
                for key in sim1.catalogue.data.keys():
                    if(key == "SRC_NAME"):
                        this_source[key] = pref1+sim1.catalogue.data[key][src]
                    else:
                        this_source[key] = sim1.catalogue.data[key][src]
                res.catalogue.addSource(this_source)
        
            print " Catalogue 2..."

            for src in range(sim2.catalogue.nsources):
                this_source = {}
                for key in sim2.catalogue.data.keys():
                    if(key == "SRC_NAME"):
                        this_source[key] = pref2+sim2.catalogue.data[key][src]
                    elif(key == "SRC_ID"):
                        this_source[key] = id_max+sim2.catalogue.data[key][src]
                    else:
                        this_source[key] = sim2.catalogue.data[key][src]
                res.catalogue.addSource(this_source)


    if(specmerge == 1):
        print " Spectrum 1..."
        res.spectrum.data = sim1.spectrum.data
        res.spectrum.nspectra = sim1.spectrum.nspectra
    if(specmerge == 2):
        print " Spectrum 2..."
        res.spectrum.data = sim2.spectrum.data
        res.spectrum.nspectra = sim2.spectrum.nspectra
    if(specmerge == 3):
        print " Spectrum 1 & 2..."
        ## ! It is much faster to concatenate
        ##   by hand than adding the spectrum 1 by 1
        ##
        ## Prefixes are not applied for spectra
        ## (because referenced in the SPECTRUM column of catalogue)

        res.spectrum.nspectra = sim1.spectrum.nspectra + sim2.spectrum.nspectra
        res.spectrum.data["ENERGY"] = np.vstack([sim1.spectrum.data["ENERGY"],sim2.spectrum.data["ENERGY"]])
        res.spectrum.data["FLUX"] = np.vstack([sim1.spectrum.data["FLUX"],sim2.spectrum.data["FLUX"]])

        if("NAME" in spec_col):
            res.spectrum.data["NAME"] = np.append(sim1.spectrum.data["NAME"],sim2.spectrum.data["NAME"])


    sim1 = 0
    sim2 = 0
    res.writeto(outfile)
    return res
#=====================================
def trimcat(infile,flaglist,outfile,nbands=1):
    """
    Trims a simput file according to a flaglist of the form
    SRC_NAME  Band1_flag  Band2_flag ...

    It returns a simput file containing only
    sources which have flags=0 in *all* bands
          
    v0.1: THERE MUST BE a 1-to-1 correspondence between
    sources and spectra in the SIMPUT file !!!!!!!!!!!!

    Author : NC, 03.07.2012

    """
    names = []
    flags = []

    #~~ Read flag file and construct keep flags ~~
    lines = file(flaglist).readlines()
    for line in lines:
        if(line[0] == '#'):
            continue
        items = line.split('|')
        items = map(string.strip,items)
        names.append(items[0])
        dummylist = []
        for j in range(nbands):
            dummylist.append(int(items[j+1]))
        flags.append(dummylist)


    names = np.array(names)
    flags = 1-np.array(flags)

    nsrc = np.shape(flags)[0]

    keepflag = np.zeros(nsrc)+1

    for i in range(nbands):
        keepflag *= flags[:,i]

    okflag = (keepflag == 1)
    keepnames = names[okflag]

    #~~ Read simput file and select the good rows ~~
    cata = simput.readfile(infile)
    ncat = cata.catalogue.nsources
    speclen = cata.spectrum.speclength
    subset = simput.simputobj(['catalogue','spectrum'],cat_columns=['SRC_NAME'],
                              spec_columns=['NAME'],speclength=speclen)

    # Associate row number to name
    allnames = cata.catalogue.data["SRC_NAME"]
    keepme = [x in keepnames for x in allnames]
    keepme = np.array(keepme) 
    
    # Construct the file
    nkeep = np.sum(keepme)
    print "Keep : %i sources" % nkeep
    
    subset.catalogue.nsources = nkeep
    for key in subset.catalogue.data.keys():
        subset.catalogue.data[key] = cata.catalogue.data[key][keepme]

    subset.spectrum.nspectra = nkeep
    for key in subset.spectrum.data.keys():
        if(key == "ENERGY" or key == "FLUX"):
            subset.spectrum.data[key] = cata.spectrum.data[key][keepme,:]
        else:
            subset.spectrum.data[key] = cata.spectrum.data[key][keepme]

    subset.writeto(outfile)

#=====================================
def subset_region(simputfile,outfile,area_def=[],area_type=''):
    
    """
    Creates a subset of a simput file with a selection according to a region

    NC, July 2013
    
    """

    # ~ Check geometry ~
    if(area_type == 'angularbox'):
        ramin = area_def[0][0]*pi/180.
        ramax = area_def[0][1]*pi/180.
        decmin = area_def[1][0]*pi/180.
        decmax = area_def[1][1]*pi/180.
        racen = 0.5*(ramin+ramax)
        deccen = 0.5*(decmin+decmax)
        area_sr = (ramax-ramin)*(cos(pi/2.-decmax)-cos(pi/2.-decmin))
        area = 3600.*area_sr*(180./pi)**2 ## arcmin2
    elif(area_type == 'projcircle'):
        racen = area_def[0]
        deccen = area_def[1]
        radius = area_def[2]
        area = pi*radius**2
        area_sr = area*(pi/180.)**2.
        area *= 3600. ## arcmin2
    else:
        print "Error: must specify area_type"
        sys.exit(1)

    # ~ Read SIMPUT catalogue ~
    incat = simput.readfile(simputfile)

    
    # ~ Make position selection ~
    if(area_type == 'angularbox'):
       okpos = (incat.catalogue.data["RA"] >= ramin)*(incat.catalogue.data["RA"] <= ramax)*(incat.catalogue.data["DEC"] >= decmin)*(incat.catalogue.data["DEC"] <= decmax)
    if(area_type == 'projcircle'):
       allra = incat.catalogue.data["RA"].copy()
       alldec = incat.catalogue.data["DEC"].copy()
       mydist = mydist_on_sphere(allra,alldec,racen,deccen)
       okpos = (mydist <= radius)
    print "Kept %i/%i after positional selection" % (np.sum(okpos),incat.catalogue.nsources)

    nkept = np.sum(okpos)

    # ~ Make subset ~
    subset = simput.simputobj(['catalogue','spectrum'],cat_columns=['SRC_NAME'],
                                 spec_columns=['NAME'],speclength=incat.spectrum.speclength)
    
    subset.catalogue.nsources = nkept
    for key in subset.catalogue.data.keys():
        subset.catalogue.data[key] = incat.catalogue.data[key][okpos]

    subset.spectrum.nspectra = nkept
    for key in subset.spectrum.data.keys():
        if(key == "ENERGY" or key == "FLUX"):
            subset.spectrum.data[key] = incat.spectrum.data[key][okpos,:]
        else:
            subset.spectrum.data[key] = incat.spectrum.data[key][okpos]

    subset.writeto(outfile)

#=====================================
def shuffle_radec(simputfile,outfile):
    
    """
    Shuffles the RA, DEC positions in a SIMPUT file

    NC, June 2015
    
    """

    cat = simput.readfile(simputfile)
    print "Shuffle %i RA, DEC positions..." % (cat.catalogue.nsources)
    random.shuffle(cat.catalogue.data["RA"])
    random.shuffle(cat.catalogue.data["DEC"])
    
    cat.writeto(outfile)
    print "Shuffled catalogue written in %s" % (outfile)
